# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'new_interface.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from Custom_Widgets.QCustomQStackedWidget import QCustomQStackedWidget
from Custom_Widgets.QCustomSlideMenu import QCustomSlideMenu
from Custom_Widgets.QCustomQPushButton import QCustomQPushButton
from Custom_Widgets.Theme import QPushButton
from Custom_Widgets.Theme import QLabel

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(716, 457)
        MainWindow.setStyleSheet(u"background:white")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.close_window_button = QPushButton(self.centralwidget)
        self.close_window_button.setObjectName(u"close_window_button")
        icon = QIcon()
        icon.addFile(u":/feather/icons/feather/window_close.png", QSize(), QIcon.Normal, QIcon.Off)
        self.close_window_button.setIcon(icon)

        self.gridLayout_2.addWidget(self.close_window_button, 0, 3, 1, 1)

        self.restore_window_button = QPushButton(self.centralwidget)
        self.restore_window_button.setObjectName(u"restore_window_button")
        icon1 = QIcon()
        icon1.addFile(u":/feather/icons/feather/maximize-2.png", QSize(), QIcon.Normal, QIcon.Off)
        self.restore_window_button.setIcon(icon1)

        self.gridLayout_2.addWidget(self.restore_window_button, 0, 2, 1, 1)

        self.minimize_window_button = QPushButton(self.centralwidget)
        self.minimize_window_button.setObjectName(u"minimize_window_button")
        icon2 = QIcon()
        icon2.addFile(u":/feather/icons/feather/window_minimize.png", QSize(), QIcon.Normal, QIcon.Off)
        self.minimize_window_button.setIcon(icon2)

        self.gridLayout_2.addWidget(self.minimize_window_button, 0, 1, 1, 1)

        self.open_close_side_bar_btn = QCustomQPushButton(self.centralwidget)
        self.open_close_side_bar_btn.setObjectName(u"open_close_side_bar_btn")
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.open_close_side_bar_btn.sizePolicy().hasHeightForWidth())
        self.open_close_side_bar_btn.setSizePolicy(sizePolicy)
        self.open_close_side_bar_btn.setMinimumSize(QSize(35, 0))
        self.open_close_side_bar_btn.setMaximumSize(QSize(35, 16777215))
        icon3 = QIcon()
        icon3.addFile(u":/feather/icons/feather/chevrons-left.png", QSize(), QIcon.Normal, QIcon.Off)
        self.open_close_side_bar_btn.setIcon(icon3)

        self.gridLayout_2.addWidget(self.open_close_side_bar_btn, 0, 0, 1, 1)


        self.gridLayout.addLayout(self.gridLayout_2, 0, 2, 1, 1)

        self.menu_widget = QCustomSlideMenu(self.centralwidget)
        self.menu_widget.setObjectName(u"menu_widget")
        sizePolicy1 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.menu_widget.sizePolicy().hasHeightForWidth())
        self.menu_widget.setSizePolicy(sizePolicy1)
        self.menu_widget.setMinimumSize(QSize(42, 0))
        self.menu_widget.setMaximumSize(QSize(42, 16777215))
        self.verticalLayout = QVBoxLayout(self.menu_widget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.page1ss = QPushButton(self.menu_widget)
        self.page1ss.setObjectName(u"page1ss")
        sizePolicy2 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.page1ss.sizePolicy().hasHeightForWidth())
        self.page1ss.setSizePolicy(sizePolicy2)
        icon4 = QIcon()
        icon4.addFile(u":/material_design/icons/material_design/chat.png", QSize(), QIcon.Normal, QIcon.Off)
        self.page1ss.setIcon(icon4)

        self.verticalLayout.addWidget(self.page1ss)

        self.page2ssss = QPushButton(self.menu_widget)
        self.page2ssss.setObjectName(u"page2ssss")
        sizePolicy2.setHeightForWidth(self.page2ssss.sizePolicy().hasHeightForWidth())
        self.page2ssss.setSizePolicy(sizePolicy2)
        icon5 = QIcon()
        icon5.addFile(u":/feather/icons/feather/edit-2.png", QSize(), QIcon.Normal, QIcon.Off)
        self.page2ssss.setIcon(icon5)

        self.verticalLayout.addWidget(self.page2ssss)

        self.pushButton = QPushButton(self.menu_widget)
        self.pushButton.setObjectName(u"pushButton")

        self.verticalLayout.addWidget(self.pushButton)

        self.verticalSpacer = QSpacerItem(20, 300, QSizePolicy.Minimum, QSizePolicy.Maximum)

        self.verticalLayout.addItem(self.verticalSpacer)


        self.gridLayout.addWidget(self.menu_widget, 2, 0, 1, 1)

        self.myStackedWidget = QCustomQStackedWidget(self.centralwidget)
        self.myStackedWidget.setObjectName(u"myStackedWidget")
        sizePolicy1.setHeightForWidth(self.myStackedWidget.sizePolicy().hasHeightForWidth())
        self.myStackedWidget.setSizePolicy(sizePolicy1)
        self.myStackedWidget.setMinimumSize(QSize(650, 400))
        self.myStackedWidget.setStyleSheet(u"\n"
"QWidget {\n"
"    background-color: white;\n"
"}\n"
"")
        self.page_2 = QWidget()
        self.page_2.setObjectName(u"page_2")
        self.myStackedWidget.addWidget(self.page_2)
        self.page = QWidget()
        self.page.setObjectName(u"page")
        self.gridLayout_3 = QGridLayout(self.page)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.html_chat = QTextEdit(self.page)
        self.html_chat.setObjectName(u"html_chat")
        self.html_chat.setMinimumSize(QSize(0, 547))
        self.html_chat.setMaximumSize(QSize(16777215, 16777215))
        self.html_chat.setStyleSheet(u"            QTextEdit {\n"
"          \n"
"                border: 1px solid #E0E0E0;\n"
"                padding: 10px;\n"
"                border-radius: 10px;\n"
"                background-color: #F7F7F7;\n"
"                color: black;  /* Cor do texto preto */\n"
"                font-family: Arial;\n"
"                font-size: 14px;\n"
"            }")

        self.gridLayout_3.addWidget(self.html_chat, 0, 0, 1, 4)

        self.verticalSpacer_2 = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Minimum)

        self.gridLayout_3.addItem(self.verticalSpacer_2, 1, 1, 1, 1)

        self.atach_file = QCustomQPushButton(self.page)
        self.atach_file.setObjectName(u"atach_file")
        self.atach_file.setStyleSheet(u"            QPushButton {\n"
"                background-color: #F7F7F7;\n"
"                border: 1px solid #E0E0E0;\n"
"                border-radius: 13px;  /* Borda arredondada (c\u00edrculo) */\n"
"                color: black;  /* Cor do texto alterada para preto */\n"
"                font-size: 13px;  /* Ajuste do tamanho do \u00edcone */\n"
"            }\n"
"            QPushButton:hover {\n"
"                background-color: #EDEDED;\n"
"            }\n"
"            QPushButton:pressed {\n"
"                background-color: #DCDCDC;\n"
"            }")
        icon6 = QIcon()
        icon6.addFile(u":/font_awesome_solid/icons/font_awesome/solid/file-arrow-down.png", QSize(), QIcon.Normal, QIcon.Off)
        self.atach_file.setIcon(icon6)
        self.atach_file.setIconSize(QSize(22, 26))

        self.gridLayout_3.addWidget(self.atach_file, 2, 0, 1, 1)

        self.mensage_input = QTextEdit(self.page)
        self.mensage_input.setObjectName(u"mensage_input")
        self.mensage_input.setMinimumSize(QSize(0, 50))
        self.mensage_input.setMaximumSize(QSize(16777215, 16777215))
        self.mensage_input.setStyleSheet(u"            QTextEdit {\n"
"                border: 1px solid #E0E0E0;\n"
"                padding: 10px;\n"
"                border-radius: 10px;\n"
"                background-color: #F7F7F7;\n"
"                color: black;  /* Cor do texto alterada para preto */\n"
"            }")

        self.gridLayout_3.addWidget(self.mensage_input, 2, 1, 1, 1)

        self.send_mensage = QCustomQPushButton(self.page)
        self.send_mensage.setObjectName(u"send_mensage")
        self.send_mensage.setStyleSheet(u"            QPushButton {\n"
"                background-color: #F7F7F7;\n"
"                border: 1px solid #E0E0E0;\n"
"                border-radius: 13px;  /* Borda arredondada (c\u00edrculo) */\n"
"                color: black;  /* Cor do texto alterada para preto */\n"
"                font-size: 16px;  /* Ajuste do tamanho do \u00edcone */\n"
"            }\n"
"            QPushButton:hover {\n"
"                background-color: #EDEDED;\n"
"            }\n"
"            QPushButton:pressed {\n"
"                background-color: #DCDCDC;\n"
"            }")
        icon7 = QIcon()
        icon7.addFile(u":/feather/icons/feather/send.png", QSize(), QIcon.Normal, QIcon.Off)
        self.send_mensage.setIcon(icon7)
        self.send_mensage.setIconSize(QSize(27, 27))

        self.gridLayout_3.addWidget(self.send_mensage, 2, 2, 1, 1)

        self.pushButton_6 = QCustomQPushButton(self.page)
        self.pushButton_6.setObjectName(u"pushButton_6")
        self.pushButton_6.setStyleSheet(u"            QPushButton {\n"
"                background-color: #F7F7F7;\n"
"                border: 1px solid #E0E0E0;\n"
"                border-radius: 13px;  /* Borda arredondada (c\u00edrculo) */\n"
"                color: black;  /* Cor do texto alterada para preto */\n"
"                font-size: 16px;  /* Ajuste do tamanho do \u00edcone */\n"
"            }\n"
"            QPushButton:hover {\n"
"                background-color: #EDEDED;\n"
"            }\n"
"            QPushButton:pressed {\n"
"                background-color: #DCDCDC;\n"
"            }")
        icon8 = QIcon()
        icon8.addFile(u":/feather/icons/feather/mic.png", QSize(), QIcon.Normal, QIcon.Off)
        self.pushButton_6.setIcon(icon8)
        self.pushButton_6.setIconSize(QSize(27, 27))

        self.gridLayout_3.addWidget(self.pushButton_6, 2, 3, 1, 1)

        self.verticalSpacer_4 = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Minimum)

        self.gridLayout_3.addItem(self.verticalSpacer_4, 3, 1, 1, 1)

        self.myStackedWidget.addWidget(self.page)

        self.gridLayout.addWidget(self.myStackedWidget, 2, 1, 1, 2)

        self.horizontalSpacer = QSpacerItem(580, 20, QSizePolicy.Maximum, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer, 0, 0, 1, 2)

        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)

        self.myStackedWidget.setCurrentIndex(1)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.close_window_button.setText("")
        self.restore_window_button.setText("")
        self.minimize_window_button.setText("")
        self.open_close_side_bar_btn.setText("")
        self.page1ss.setText("")
        self.page2ssss.setText("")
        self.pushButton.setText("")
        self.atach_file.setText("")
        self.mensage_input.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Mensagem...", None))
        self.send_mensage.setText("")
        self.pushButton_6.setText("")
    # retranslateUi




######################################### SoftwareAI Core #########################################

# IMPORT SoftwareAI Functions
from softwareai.CoreApp._init_functions_ import *
#########################################
# IMPORT SoftwareAI Functions Submit Outputs
from softwareai.CoreApp._init_submit_outputs_ import _init_output_
#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Keys 
from softwareai.CoreApp._init_keys_ import *
#########################################
# IMPORT Formatmessage
from softwareai.CoreUi.Chat.Chat.Formatmessage import format_message
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *
#########################################

from pydantic import BaseModel
from firebase_admin import App

import importlib

class TitleAndPreface(BaseModel):
    title: str
    preface: str



class FirebaseKeysinit:
    @staticmethod
    def _init_app_(name_app):
        module_path = "softwareai.CoreApp.KeysFirebase.keys"
        
        # Importa o módulo dinamicamente
        keys_module = importlib.import_module(module_path)

        # Verifica se o atributo (função ou classe) existe no módulo
        if hasattr(keys_module, name_app):
            # Obtém o atributo com base no nome dinâmico
            imported_name_app = getattr(keys_module, name_app)

            # Verifica se é uma função ou classe e chama ou instância
            if callable(imported_name_app):
                appfb = imported_name_app()  # Chama a função
            else:
                appfb = imported_name_app  # Atribui diretamente se for outro tipo (como uma classe)

            return appfb
        else:
            raise AttributeError(f"O módulo '{module_path}' não contém o atributo '{name_app}'.")

class OpenAIKeysinit:
    
    def _init_client_(key_api):
        client = OpenAI(
            api_key=key_api,
        )
        return client


class AutenticateAgent:

    def create_or_auth_vectorstoreadvanced(key, UseVectorstoreToGenerateFiles, app1):
        """
        This function checks if a user's vectorstore advanced settings exist in the database.

        Parameters:
        ----------
        key : str
            The unique identifier for the user.
        UseVectorstoreToGenerateFiles : bool
            The boolean indicating whether vectorstore advanced features should be used.

        Returns:
        --------
        str
            A string representing the value of the 'UseVectorstoreToGenerateFiles' setting.

        Raises:
        -------
        Exception
            If an error occurs during the process.

        Example:
        --------
        >>> create_or_auth_vectorstoreadvanced('user123', True)
        'True'

        Note:
        -----
        - The function first attempts to retrieve the current 'UseVectorstoreToGenerateFiles' setting for the specified user from the database.
        - If the setting does not exist, it adds the setting to the database with the provided value and returns that value.
        
        """

        try:
            ref1 = db.reference(f'ai_org_vectorstoreadvanced/User_{key}', app=app1)
            data1 = ref1.get()
            UseVectorstoreToGenerateFiles = data1['UseVectorstoreToGenerateFiles']
            return str(UseVectorstoreToGenerateFiles)
        except Exception as err234:
            ref1 = db.reference(f'ai_org_vectorstoreadvanced', app=app1)
            controle_das_funcao2 = f"User_{key}"
            controle_das_funcao_info_2 = {
                "UseVectorstoreToGenerateFiles": f'{UseVectorstoreToGenerateFiles}'
            }
            ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)
            return str(UseVectorstoreToGenerateFiles)

    def create_or_auth_AI(
        app1,
        client,
        key: str, 
        instructionsassistant: Optional[str] = None,
        nameassistant: Optional[str] = None, 
        model_select: Optional[str] = "gpt-4o-mini-2024-07-18", 
        tools: Optional[List] = [{"type": "file_search"},{"type": "code_interpreter"}],
        vectorstore: Optional[List] = None,
        codeinterpreter: Optional[List] = None,
        response_format: Optional[str] = "text",

        
        ):
        """ 
        :param key: this is the key that represents the agent in the database
            
        :param instructionsassistant: This argument is the instruction of the agent's behavior The maximum length is 256,000 characters.
        
        :param nameassistant: This argument is the name of the agent The maximum length is 256 characters.
        
        :param model_select: This argument is the AI model that the agent will use
            
        :param tools: This argument is the agent's tools  There can be a maximum of 128 tools per assistant. Tools can be of types code_interpreter, file_search, vectorstore, or function.
            
        :param vectorstore: This argument is the vector storage id desired when creating or authenticating the agent
        response_format: Optional[str] = "json_object",
        response_format: Optional[str] = "json_schema_TitleAndPreface",
        response_format: Optional[str] = "text",
        """

        
        try:
            ref1 = db.reference(f'ai_org_assistant_id/User_{key}', app=app1)
            data1 = ref1.get()
            assistant_iddb = data1['assistant_id']
            instructionsassistantdb = data1['instructionsassistant']
            nameassistantdb = data1['nameassistant']
            model_selectdb = data1['model_select']
            
            if instructionsassistant:
                client.beta.assistants.update(
                    assistant_id=str(assistant_iddb),
                    instructions=instructionsassistant
                    
                )
                ref1 = db.reference(f'ai_org_assistant_id', app=app1)
                controle_das_funcao2 = f"User_{key}"
                controle_das_funcao_info_2 = {
                    "assistant_id": f'{assistant_iddb}',
                    "instructionsassistant": f'{instructionsassistant}',
                    "nameassistant": f'{nameassistantdb}',
                    "model_select": f'{model_selectdb}',
                    "tools": f'{tools}',
                    "key": f'{key}',
                }
                ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)

            if response_format == "json_object":
                client.beta.assistants.update(
                    assistant_id=str(assistant_iddb),
                    response_format={ "type": "json_object" }
                    
                )
            elif response_format == "json_schema_TitleAndPreface":
                client.beta.assistants.update(
                    assistant_id=str(assistant_iddb),
                    response_format={
                        "type": "json_schema",
                        "json_schema": {
                            "name": "book_schema",
                            "schema": {
                                "type": "object",
                                "properties": {
                                    "title": {
                                        "type": "string",
                                        "description": "Título do Livro"
                                    },
                                    "preface": {
                                        "type": "string",
                                        "description": "Texto detalhado do prefácio, com no mínimo 500 palavras."
                                    }
                                },
                                "required": [
                                    "title",
                                    "preface"
                                ],
                                "additionalProperties": False  # Deve ser booleano
                            },
                            "strict": True  # Deve ser booleano
                        }
                    }
                )
            elif response_format == "text":

                client.beta.assistants.update(
                    assistant_id=str(assistant_iddb),
                    tools=tools
                    
                )


            if vectorstore:
                client.beta.assistants.update(
                    assistant_id=str(assistant_iddb),
                    tool_resources={"file_search": {"vector_store_ids": vectorstore}},
                )

            if codeinterpreter:
                list_file_id = []
                for path in codeinterpreter:
                    file = client.files.create(
                        file=open(path, "rb"),
                        purpose='assistants'
                    )
                    list_file_id.append(file.id)
                code_interpreter_in_agent = list_file_id
                client.beta.assistants.update(
                    assistant_id=str(assistant_iddb),
                    tool_resources={
                    "code_interpreter": {
                        "file_ids": code_interpreter_in_agent
                        }
                    }
                )
                

            return str(assistant_iddb), str(instructionsassistantdb), str(nameassistantdb), str(model_selectdb)
        except Exception as err234:
            if tools:
           
                assistant = client.beta.assistants.create(
                    name=nameassistant,
                    instructions=instructionsassistant,
                    model=model_select
                )


                if response_format == "json_object":
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        response_format={ "type": "json_object" }
                        
                    )
                elif response_format == "json_schema_TitleAndPreface":
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        response_format={
                            "type": "json_schema",
                            "json_schema": {
                                "name": "book_schema",
                                "schema": {
                                    "type": "object",
                                    "properties": {
                                        "title": {
                                            "type": "string",
                                            "description": "Título do Livro"
                                        },
                                        "preface": {
                                            "type": "string",
                                            "description": "Texto detalhado do prefácio, com no mínimo 500 palavras."
                                        }
                                    },
                                    "required": [
                                        "title",
                                        "preface"
                                    ],
                                    "additionalProperties": False  # Deve ser booleano
                                },
                                "strict": True  # Deve ser booleano
                            }
                        }
                        
                    )
                elif response_format == "text":
                 
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        tools=tools
                        
                    )
                if vectorstore:
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        tool_resources={"file_search": {"vector_store_ids": vectorstore}},
                    )

                if codeinterpreter:
                    list_file_id = []
                    for path in codeinterpreter:
                        file = client.files.create(
                            file=open(path, "rb"),
                            purpose='assistants'
                        )
                        list_file_id.append(file.id)
                    code_interpreter_in_agent = list_file_id

                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        tool_resources={
                        "code_interpreter": {
                            "file_ids": code_interpreter_in_agent
                            }
                        }
                    )
            
            else:
                assistant = client.beta.assistants.create(
                    name=nameassistant,
                    instructions=instructionsassistant,
                    model=model_select,
                )
                if response_format == "json_object":
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        response_format={ "type": "json_object" }
                        
                    )
                elif response_format == "json_schema_TitleAndPreface":
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        response_format={
                            "type": "json_schema",
                            "json_schema": {
                                "name": "book_schema",
                                "schema": {
                                    "type": "object",
                                    "properties": {
                                        "title": {
                                            "type": "string",
                                            "description": "Título do Livro"
                                        },
                                        "preface": {
                                            "type": "string",
                                            "description": "Texto detalhado do prefácio, com no mínimo 500 palavras."
                                        }
                                    },
                                    "required": [
                                        "title",
                                        "preface"
                                    ],
                                    "additionalProperties": False  # Deve ser booleano
                                },
                                "strict": True  # Deve ser booleano
                            }
                        }
                        
                    )
                elif response_format == "text":
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        tools=[{"type": "file_search"}, {"type": "code_interpreter"}],
                    )

                if vectorstore:
                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        tool_resources={"file_search": {"vector_store_ids": vectorstore}},
                    )

                if codeinterpreter:
                    list_file_id = []
                    for path in codeinterpreter:
                        file = client.files.create(
                            file=open(path, "rb"),
                            purpose='assistants'
                        )
                        list_file_id.append(file.id)
                    code_interpreter_in_agent = list_file_id

                    client.beta.assistants.update(
                        assistant_id=assistant.id,
                        tool_resources={
                        "code_interpreter": {
                            "file_ids": code_interpreter_in_agent
                            }
                        }
                    )

                    

            ref1 = db.reference(f'ai_org_assistant_id', app=app1)
            controle_das_funcao2 = f"User_{key}"
            controle_das_funcao_info_2 = {
                "assistant_id": f'{assistant.id}',
                "instructionsassistant": f'{instructionsassistant}',
                "nameassistant": f'{nameassistant}',
                "model_select": f'{model_select}',
                "tools": f'{tools}',
                "key": f'{key}',
            }
            ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)
            return str(assistant.id), str(instructionsassistant), str(nameassistant), str(model_select)
        
    def create_or_auth_thread(app1, client, key, 
                            attachavectorstoretoThreads: Optional[List] = None,
                            code_interpreter_in_thread: Optional[List] = None,
                            user_id: Optional[str] = None
                            
                            ):

        if user_id is not None:
                
            try:
                ref1 = db.reference(f'ai_org_thread_Id/User_{user_id}', app=app1)
                data1 = ref1.get()
                thread_Id = data1['thread_id']
                print(thread_Id)
                if attachavectorstoretoThreads:
                    client.beta.threads.update(
                        thread_id=str(thread_Id),
                        tool_resources={
                            "file_search": {
                            "vector_store_ids": attachavectorstoretoThreads
                            }
                        }
                        
                    )
                if code_interpreter_in_thread:    
                    thread = client.beta.threads.update(
                        thread_id=str(thread_Id),
                        tool_resources={"code_interpreter": {
                            "file_ids": code_interpreter_in_thread
                            }
                        }
                    )

                return str(thread_Id)
            except Exception as err234z:
                print(err234z)
                tool_resources = {}
                if attachavectorstoretoThreads:
                    tool_resources["file_search"] = {"vector_store_ids": attachavectorstoretoThreads}

                if code_interpreter_in_thread:
                    tool_resources["code_interpreter"] = {"file_ids": code_interpreter_in_thread}


                thread = client.beta.threads.create(
                    tool_resources=tool_resources
                )

                ref1 = db.reference(f'ai_org_thread_Id', app=app1)
                controle_das_funcao2 = f"User_{user_id}"
                controle_das_funcao_info_2 = {
                    "thread_id": f'{thread.id}',
                    "user_id": f'{user_id}'
                }
                ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)

                return str(thread.id)
        
        else:
                
            try:
                ref1 = db.reference(f'ai_org_thread_Id/User_{key}', app=app1)
                data1 = ref1.get()
                thread_Id = data1['thread_id']
                print(thread_Id)
                if attachavectorstoretoThreads:
                    client.beta.threads.update(
                        thread_id=str(thread_Id),
                        tool_resources={
                            "file_search": {
                            "vector_store_ids": attachavectorstoretoThreads
                            }
                        }
                        
                    )
                if code_interpreter_in_thread:    
                    thread = client.beta.threads.update(
                        thread_id=str(thread_Id),
                        tool_resources={"code_interpreter": {
                            "file_ids": code_interpreter_in_thread
                            }
                        }
                    )

                return str(thread_Id)
            except Exception as err234z:
                print(err234z)
                tool_resources = {}
                if attachavectorstoretoThreads:
                    tool_resources["file_search"] = {"vector_store_ids": attachavectorstoretoThreads}

                if code_interpreter_in_thread:
                    tool_resources["code_interpreter"] = {"file_ids": code_interpreter_in_thread}


                thread = client.beta.threads.create(
                    tool_resources=tool_resources
                )

                ref1 = db.reference(f'ai_org_thread_Id', app=app1)
                controle_das_funcao2 = f"User_{key}"
                controle_das_funcao_info_2 = {
                    "thread_id": f'{thread.id}',
                    "key": f'{key}'
                }
                ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)

                return str(thread.id)


class ResponseAgent:

    def ResponseAgent_message_completions(prompt, 
                                        key_api, 
                                        lang: Optional[str] = "pt",
                                        model: Optional[str] = "gpt-4o-mini-2024-07-18", 
                                        Debug: Optional[bool] = True,
                                        json_format: Optional[bool] = True, 
                                        store: Optional[bool] = True, 
                                        AgentDestilation: Optional[bool] = False,
                                        AgentName: Optional[str] = None,
                                        AgentInstruction: Optional[str] = ""
                                        ):
        """
        Envia uma mensagem para o modelo de chat da OpenAI e retorna a resposta.

        Parâmetros:
        - instruction (str): O texto ao qual o assistente responderá.
        - sistema (str, opcional): Instrução de sistema para o assistente. Padrão é uma string vazia.
        - json_format (bool, opcional): Define se a resposta será JSON ou texto simples. Padrão é True.
        - store (bool, opcional): Define se a interação será armazenada.

        Retorno:
        - str: Resposta do assistente.

        Exceções:
        - Exception: Se houver erro durante a requisição à API.
        """
        def log_message(message_pt, message_en, color, bold=False):
            if Debug:
                attrs = ['bold'] if bold else []
                cprint(message_pt if lang == "pt" else message_en, color, attrs=attrs)

        url = "https://api.openai.com/v1/chat/completions"
        headers = {
            "Authorization": f"Bearer {key_api}",
            "Content-Type": "application/json"
        }

        log_message("🔑 Configurando cabeçalhos da requisição...", 
                    "🔑 Setting up request headers...", 'blue')

        formato = "text"
        if json_format:
            formato = "json_object"
        
        log_message(f'📦 Formato de resposta definido como: {formato}', 
                    f'📦 Response format set as: {formato}', 'cyan')

        mensagem = []
        if AgentInstruction != "":
            log_message(f'⚙️ Adicionando instrução de sistema: {AgentInstruction}', 
                        f'⚙️ Adding system instruction: {AgentInstruction}', 'cyan')
            mensagem.append({"role": "system", "content": AgentInstruction})

        mensagem.append({"role": "user", "content": prompt})

        log_message(f'📝 Montando mensagem do usuário: {prompt}', 
                    f'📝 Building user message: {prompt}', 'cyan')

        data = {
            "model": model,
            "messages": mensagem,
            "store": store,
            "max_tokens": 16_384,
            "response_format": {"type": formato},
        }

        log_message("🚀 Enviando requisição para a API...", 
                    "🚀 Sending request to the API...", 'blue')

        try:
            response = requests.post(url, headers=headers, json=data)
            
            if response.status_code == 200:
                log_message("✅ Requisição bem-sucedida! Processando resposta...", 
                            "✅ Request successful! Processing response...", 'green')
                
                response_json = response.json()
                if json_format:
                    log_message("📄 Retornando resposta em formato JSON.", 
                                "📄 Returning response in JSON format.", 'cyan')
                    return json.loads(response_json['choices'][0]['message']['content'])
                
                if AgentDestilation == True: 

                    log_message("📄 Storing response with Agent Distillation", 
                                "📄 Returning response in text format.", 'cyan')
                    
                    Agent_destilation.DestilationResponseAgent(prompt, response_json['choices'][0]['message']['content'], AgentInstruction, AgentName)
                                    
                log_message("📄 Retornando resposta em formato texto.", 
                            "📄 Returning response in text format.", 'cyan')
                                        

                return response_json['choices'][0]['message']['content']
            else:
                log_message(f'❌ Erro na requisição: {response.status_code}, {response.text}', 
                            f'❌ Request error: {response.status_code}, {response.text}', 'red', bold=True)
                return None

        except Exception as e:
            log_message(f'🔥 Exceção durante a requisição: {str(e)}', 
                        f'🔥 Exception during request: {str(e)}', 'red', bold=True)
            return None
        
    def ResponseAgent_message_with_assistants(
                                            mensagem: str,
                                            agent_id: str,
                                            key: str,
                                            app1, 
                                            client,
                                            app_product: Optional[App] = None,
                                            user_id: Optional[str] = None,
                                            tools: Optional[List] = None,
                                            model_select: Optional[str] = None,
                                            aditional_instructions: Optional[str] = None,
                                            streamflag: bool = False,
                                            QstreamLogger: Optional[Signal] = None,
                                            QstreamLoggerCode: Optional[Signal] = None,
                                            Debug: Optional[bool] = True,
                                            DebugTokens: Optional[bool] = True,
                                            AgentDestilation: Optional[bool] = True,
                                            Moderation: Optional[bool] = False,
                                            lang: Optional[str] = "pt",
                                            Upload_multiples_file_in_thread: Optional[List[str]] = None,
                                            Upload_1_file_in_message: Optional[str] = None,
                                            Upload_1_image_for_vision_in_thread: Optional[str] = None,
                                            Upload_list_for_code_interpreter_in_thread: Optional[list] = None,
                                            Upload_1_file_for_code_interpreter_in_message: Optional[str] = None,
                                            vectorstore_in_Thread: Optional[List] = None

                                                                        
                                        ):


        def log_message(message_pt, message_en, color, bold=False):
            if Debug:
                attrs = ['bold'] if bold else []
                cprint(message_pt if lang == "pt" else message_en, color, attrs=attrs)


        if Upload_1_image_for_vision_in_thread is not None and Upload_1_image_for_vision_in_thread != "":
            log_message("🖼️ Iniciando upload de imagem para visão computacional...", "🖼️ Starting image upload for computer vision...", "blue")
            for i in range(8):
                    
                code_interpreter_in_thread = None
                with open(Upload_1_image_for_vision_in_thread, "rb") as image_file:
                    file = client.files.create(file=image_file, purpose="vision")

                    log_message(f"📤 Imagem enviada com sucesso. ID do arquivo: {file.id}",
                                f"📤 Image uploaded successfully. File ID: {file.id}", "green")
                    if user_id is not None:
                        threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread, user_id)
                    else:
                        threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread)

                    log_message(f"🧵 Thread criada/autenticada. ID da thread: {threead_id}",
                                f"🧵 Thread created/authenticated. Thread ID: {threead_id}", "cyan")

                    try:
                        message = client.beta.threads.messages.create(
                            thread_id=threead_id,
                            role="user",
                            content=[
                                {"type": "text", "text": f"""{mensagem}"""},
                                {"type": "image_file", "image_file": {"file_id": file.id}}
                            ]
                        )

                        log_message("✅ Mensagem com imagem enviada com sucesso.",
                                    "✅ Message with image sent successfully.", "green")
                        break
                    except Exception as e:
                        try:
                            ref1 = db.reference(f'ai_org_thread_Id/User_{key}', app=app1)
                            ref1.delete()                                
                            client.beta.threads.delete(threead_id)
                        except Exception as e2:
                            print(e2)
                        continue

        elif Upload_1_file_in_message is not None :
            log_message("📄 Iniciando upload de arquivo para a mensagem...",
                        "📄 Starting file upload for the message...", "blue")
            for i in range(8):
                    
                code_interpreter_in_thread = None
                message_file = client.files.create(file=open(Upload_1_file_in_message, "rb"), purpose="assistants")

                log_message(f"📤 Arquivo enviado. ID do arquivo: {message_file.id}",
                            f"📤 File uploaded. File ID: {message_file.id}", "green")
                if user_id is not None:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread, user_id)
                else:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread)

                log_message(f"🧵 Thread criada/autenticada. ID da thread: {threead_id}",
                            f"🧵 Thread created/authenticated. Thread ID: {threead_id}", "cyan")


                try:
                    message = client.beta.threads.messages.create(
                        thread_id=threead_id,
                        role="user",
                        content=mensagem,
                        attachments=[{"file_id": message_file.id, "tools": [{"type": "file_search"}]}]
                    )
                    
                    log_message("✅ Mensagem com arquivo enviada com sucesso.",
                                "✅ Message with file sent successfully.", "green")
                    break
                except Exception as e:
                    try:
                        ref1 = db.reference(f'ai_org_thread_Id/User_{key}', app=app1)
                        ref1.delete()                                
                        client.beta.threads.delete(threead_id)
                    except Exception as e2:
                        print(e2)
                    continue

        elif Upload_multiples_file_in_thread is not None :
            log_message("📂 Iniciando upload de múltiplos arquivos para a thread...",
                        "📂 Starting upload of multiple files to the thread...", "blue")
            for i in range(8):
                    
                code_interpreter_in_thread = None
                list_as_string = json.dumps(Upload_multiples_file_in_thread)
                namehash = hashlib.sha256(list_as_string.encode()).hexdigest()
                vector_store = client.beta.vector_stores.create(name=f"Upload_{namehash[:5]}")

                log_message(f"📦 Vector store criado: {vector_store.id}",
                            f"📦 Vector store created: {vector_store.id}", "cyan")

                file_streams = [open(path, "rb") for path in Upload_multiples_file_in_thread]
                file_batch = client.beta.vector_stores.file_batches.upload_and_poll(
                    vector_store_id=vector_store.id, files=file_streams
                )

                log_message(f"📤 Arquivos enviados: {file_batch.status}",
                            f"📤 Files uploaded: {file_batch.status}", "green")
                if user_id is not None:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key,  [vector_store.id], code_interpreter_in_thread, user_id)
                else:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key,  [vector_store.id], code_interpreter_in_thread)

                log_message(f"🧵 Thread criada/autenticada. ID da thread: {threead_id}",
                            f"🧵 Thread created/authenticated. Thread ID: {threead_id}", "cyan")


                try:
                    message = client.beta.threads.messages.create(thread_id=threead_id, role="user", content=mensagem)

                        
                    log_message("✅ Mensagem com múltiplos arquivos enviada com sucesso.",
                                "✅ Message with multiple files sent successfully.", "green")

                    break
                except Exception as e:
                    try:
                        ref1 = db.reference(f'ai_org_thread_Id/User_{key}', app=app1)
                        ref1.delete()                                
                        client.beta.threads.delete(threead_id)
                    except Exception as e2:
                        print(e2)
                    continue

        elif Upload_list_for_code_interpreter_in_thread is not None:
            log_message("📁 Iniciando upload de arquivos para o interpretador de código...",
                        "📁 Starting file upload for the code interpreter...", "blue")
            for i in range(8):
                        
                list_file_id = []
                for path in Upload_list_for_code_interpreter_in_thread:
                    file = client.files.create(file=open(path, "rb"), purpose='assistants')
                    list_file_id.append(file.id)

                    log_message(f"📤 Arquivo enviado. ID: {file.id}",
                                f"📤 File uploaded. ID: {file.id}", "green")

                code_interpreter_in_thread = list_file_id
                if user_id is not None:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread, user_id)
                else:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread)

                log_message(f"🧵 Thread criada/autenticada. ID da thread: {threead_id}",
                            f"🧵 Thread created/authenticated. Thread ID: {threead_id}", "cyan")

                try:
                    message = client.beta.threads.messages.create(thread_id=threead_id, role="user", content=mensagem)
                    log_message("✅ Mensagem com arquivos para o interpretador de código enviada com sucesso.",
                                "✅ Message with files for the code interpreter sent successfully.", "green")
                    break
                except Exception as e:
                    try:
                        ref1 = db.reference(f'ai_org_thread_Id/User_{key}', app=app1)
                        ref1.delete()                                
                        client.beta.threads.delete(threead_id)
                    except Exception as e2:
                        print(e2)
                    continue

        elif Upload_1_file_for_code_interpreter_in_message is not None:
            log_message("📄 Iniciando upload de arquivo para o interpretador de código na mensagem...",
                        "📄 Starting file upload for the code interpreter in the message...", "blue")
            for i in range(8):
                            
                code_interpreter_in_thread = None
                file = client.files.create(file=open(Upload_1_file_for_code_interpreter_in_message, "rb"), purpose='assistants')

                log_message(f"📤 Arquivo enviado. ID: {file.id}",
                            f"📤 File uploaded. ID: {file.id}", "green")
                if user_id is not None:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread, user_id)
                else:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread)

                log_message(f"🧵 Thread criada/autenticada. ID da thread: {threead_id}",
                            f"🧵 Thread created/authenticated. Thread ID: {threead_id}", "cyan")


                try:
                    message = client.beta.threads.messages.create(
                        thread_id=threead_id,
                        role="user",
                        content=mensagem,
                        attachments=[{"file_id": file.id, "tools": [{"type": "code_interpreter"}]}]
                    )
                    log_message("✅ Mensagem com arquivo para interpretador de código enviada com sucesso.",
                                "✅ Message with file for the code interpreter sent successfully.", "green")

                    break
                except Exception as e:
                    try:
                        ref1 = db.reference(f'ai_org_thread_Id/User_{key}', app=app1)
                        ref1.delete()                                
                        client.beta.threads.delete(threead_id)
                    except Exception as e2:
                        print(e2)
                    continue

        else:
            log_message("📝 Enviando mensagem sem anexos...",
                        "📝 Sending message without attachments...", "blue")
            for i in range(8):
                                
                code_interpreter_in_thread = None
                if user_id is not None:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread, user_id)
                else:
                    threead_id = AutenticateAgent.create_or_auth_thread(app1, client, key, vectorstore_in_Thread, code_interpreter_in_thread)

                log_message(f"🧵 Thread criada/autenticada. ID da thread: {threead_id}",
                            f"🧵 Thread created/authenticated. Thread ID: {threead_id}", "cyan")


                try:

                    message = client.beta.threads.messages.create(thread_id=threead_id, role="user", content=mensagem)
                    log_message("✅ Mensagem enviada com sucesso.",
                                "✅ Message sent successfully.", "green")
                    break
                except Exception as e:
                    try:
                        ref1 = db.reference(f'ai_org_thread_Id/User_{key}', app=app1)
                        ref1.delete()                                
                        client.beta.threads.delete(threead_id)
                    except Exception as e2:
                        print(e2)
                    continue


                        
        code_buffer = None 
        formatted_output = ""

        if streamflag == True:
            log_message('🚀 Iniciando execução em modo streaming...',
                        '🚀 Starting execution in streaming mode...', 'blue')

            accumulated_text = ""
            total_text = ""

            log_message('📡 Conectando ao stream...',
                        '📡 Connecting to the stream...', 'cyan')


            with client.beta.threads.runs.stream(
                thread_id=threead_id,
                assistant_id=agent_id,
                event_handler=EventHandler(),
                tools=tools,
                additional_instructions=aditional_instructions,
                model=model_select
            ) as stream:

                log_message('✅ Conectado ao stream. Recebendo dados...',
                            '✅ Connected to the stream. Receiving data...', 'green')

                for text in stream.text_deltas:

                    total_text += text
                    if QstreamLogger is not None:
                        accumulated_text += text
                        while len(accumulated_text) >= 100:
                            match = re.search(r"[ \n.,!?]+", accumulated_text[100:])
                            cut_index = 100 + match.start() if match else len(accumulated_text)
                            chunk = accumulated_text[:cut_index]
                            accumulated_text = accumulated_text[cut_index:]

                            log_message(f'✂️ Processando chunk: {chunk[:50]}...',
                                        f'✂️ Processing chunk: {chunk[:50]}...', 'yellow')

                            if code_buffer is not None:
                                code_buffer += chunk
                                if "```" in code_buffer:
                                    code_content, _, remainder = code_buffer.partition("```")
                                    QstreamLoggerCode.emit(code_content.strip())

                                    log_message(f'💻 Código detectado no buffer: {code_content[:50]}...',
                                                f'💻 Code detected in the buffer: {code_content[:50]}...', 'magenta')

                                    formatted_output += '<pre style="margin: 0; padding: 0; white-space: pre-wrap; background-color: #F7F7F7; color: #0e6303;"><b>[CÓDIGO NO QUADRO]</b>'
                                    code_buffer = None
                                    chunk = remainder
                                else:
                                    continue

                            while "```python" in chunk:
                                pre_code, _, rest = chunk.partition("```python")

                                log_message('🔎 Bloco de código Python detectado.',
                                            '🔎 Python code block detected.', 'magenta')

                                formatted_output += ""  # format_message(pre_code)
                                chunk = rest

                                if "```" in chunk:
                                    code_content, _, remainder = chunk.partition("```")
                                    QstreamLoggerCode.emit(code_content.strip())

                                    log_message(f'📄 Emitindo código Python: {code_content[:50]}...',
                                                f'📄 Emitting Python code: {code_content[:50]}...', 'magenta')

                                    formatted_output += ""  # python_functions.ignore_python_code(code_content)
                                    chunk = remainder
                                else:
                                    code_buffer = chunk
                                    break

                            formatted_output += format_message(chunk)
                            QstreamLogger.emit(
                                f'<div style="display: flex; justify-content: flex-start;">'
                                f'<div style="color: black; padding: 8px; border-radius: 8px; margin: 5px; max-width: 70%;">'
                                f'{formatted_output}</div></div>'
                            )

                            log_message('📤 Chunk emitido para a interface.',
                                        '📤 Chunk sent to the interface.', 'green')

                            formatted_output = ""


                        if accumulated_text:
                            log_message('📄 Processando texto restante no buffer...',
                                        '📄 Processing remaining text in buffer...', 'cyan')

                            formatted_output += format_message(accumulated_text)
                            QstreamLogger.emit(
                                f'<div style="display: flex; justify-content: flex-start;">'
                                f'<div style="color: black; padding: 8px; border-radius: 8px; margin: 5px; max-width: 70%;">'
                                f'{formatted_output}</div></div>'
                            )

                    else:
                        sys.stdout.write(text)
                        sys.stdout.flush()  # Garante que o texto seja exibido imediatamente

                log_message(f'✅ Transmissão concluída. Total de texto processado: {len(total_text)} caracteres.',
                            f'✅ Transmission completed. Total text processed: {len(total_text)} characters.', 'green')

                return total_text, 0, 0, 0
       
        elif streamflag == False:
            log_message('🚀 Iniciando execução sem streaming...', '🚀 Starting execution without streaming...', 'blue')

            if tools:
                if aditional_instructions and model_select:
                    log_message('🛠️ Executando com ferramentas, instruções adicionais e modelo selecionado.',
                                '🛠️ Running with tools, additional instructions, and selected model.', 'blue')
                    run = client.beta.threads.runs.create(
                        thread_id=threead_id,
                        assistant_id=agent_id,
                        tools=tools,
                        additional_instructions=aditional_instructions,
                        model=model_select,
                    )
                else:
                    log_message('🛠️ Executando com ferramentas, sem instruções adicionais.',
                                '🛠️ Running with tools, without additional instructions.', 'blue')
                    run = client.beta.threads.runs.create(
                        thread_id=threead_id,
                        assistant_id=agent_id,
                        tools=tools
                    )
            else:
                if aditional_instructions and model_select:
                    log_message('📄 Executando sem ferramentas, com instruções adicionais e modelo selecionado.',
                                '📄 Running without tools, with additional instructions and selected model.', 'blue')
                    run = client.beta.threads.runs.create(
                        thread_id=threead_id,
                        assistant_id=agent_id,
                        additional_instructions=aditional_instructions,
                        model=model_select,
                    )
                else:
                    log_message('📄 Executando sem ferramentas e sem instruções adicionais.',
                                '📄 Running without tools and without additional instructions.', 'blue')
                    run = client.beta.threads.runs.create(
                        thread_id=threead_id,
                        assistant_id=agent_id
                    )

            contador = 0
            log_message('⏳ Monitorando status da execução...', '⏳ Monitoring execution status...', 'cyan')
            i = 0
            for irg in range(900):
                time.sleep(2)
                
                run_status = client.beta.threads.runs.retrieve(
                    thread_id=threead_id,
                    run_id=run.id
                )
    
                if run_status.status == 'requires_action':
                    log_message('⚙️ Ação requerida. Processando chamadas de ferramentas...',
                                '⚙️ Action required. Processing tool calls...', 'yellow')
                    for tool_call in run_status.required_action.submit_tool_outputs.tool_calls:
                        if tool_call.type == 'function':
                            function_name = tool_call.function.name
                            function_arguments = tool_call.function.arguments

                            log_message(f'🔧 Função chamada: {function_name}',
                                        f'🔧 Function called: {function_name}', 'yellow')
                            log_message(f'📥 Argumentos: {function_arguments}',
                                        f'📥 Arguments: {function_arguments}', 'yellow')
                            log_message(f'🆔 Tool Call ID: {tool_call.id}',
                                        f'🆔 Tool Call ID: {tool_call.id}', 'yellow')


                            if app_product:
                                _init_output_(
                                    function_name,
                                    function_arguments,
                                    tool_call,
                                    threead_id,
                                    client,
                                    run,
                                    app1,
                                    OpenAIKeysinit,
                                    OpenAIKeysteste,
                                    GithubKeys,
                                    python_functions,
                                    Agent_files_update,
                                    AutenticateAgent,
                                    ResponseAgent,
                                    app_product,
                                )
                            else:
                                _init_output_(
                                    function_name,
                                    function_arguments,
                                    tool_call,
                                    threead_id,
                                    client,
                                    run,
                                    app1,
                                    OpenAIKeysinit,
                                    OpenAIKeysteste,
                                    GithubKeys,
                                    python_functions,
                                    Agent_files_update,
                                    AutenticateAgent,
                                    ResponseAgent,
                                )
                elif run_status.status == 'completed':
                    log_message('✅ Execução concluída com sucesso.',
                                '✅ Execution completed successfully.', 'green')
                    break
                elif run_status.status == 'failed':
                    log_message('❌ Execução falhou.',
                                '❌ Execution failed.', 'red')
                    break
                elif run_status.status == 'in_progress':
                    pontos = '.' * i 
                    log_message(f'💭 Pensando{pontos}',
                                f'💭 Thinking{pontos}', 'cyan')
                    i = i + 1 if i < 3 else 1  # Reinicia o contador após 3

                else:
                    contador += 1
                    if contador == 15:
                        log_message('⚠️ Tempo limite atingido. Finalizando monitoramento.',
                                    '⚠️ Timeout reached. Stopping monitoring.', 'red')
                        break
                    log_message('⏳ Aguardando a execução ser completada...',
                                '⏳ Waiting for execution to complete...', 'cyan')

            log_message('📨 Recuperando mensagens do thread...',
                        '📨 Retrieving messages from the thread...', 'blue')

            messages = client.beta.threads.messages.list(thread_id=threead_id)

            file_id = None
            contador23 = 0

            for message in messages:
                for mensagem_contexto in message.content:
                    log_message(f'📩 Mensagem recebida: {mensagem_contexto.text.value}',
                                f'📩 Message received: {mensagem_contexto.text.value}', 'blue')

                    valor_texto = mensagem_contexto.text.value

                    if DebugTokens:
                        price = ResponseAgent.calculate_dollar_value(run_status.usage.prompt_tokens, run_status.usage.completion_tokens)
                        if lang == "en":
                            log_message(f'📜 Tokens consumed : {run_status.usage.total_tokens} 💸${price:.4f}',
                                        f'📜 Tokens consumed : {run_status.usage.total_tokens} 💸${price:.4f}', 'yellow', bold=True)
                        elif lang == "pt":
                            log_message(f'📜 Tokens Consumidos: {run_status.usage.total_tokens} 💸 ${price:.4f}',
                                        f'📜 Tokens Consumidos: {run_status.usage.total_tokens} 💸 ${price:.4f}', 'yellow', bold=True)

                    if AgentDestilation == True: 

                        log_message("📄 Storing response with Agent Distillation", 
                                    "📄 Returning response in text format.", 'cyan')
                        
                        ref1 = db.reference(f'ai_org_assistant_id/User_{key}', app=app1)
                        data1 = ref1.get()
                        instructionsassistant = data1['instructionsassistant']
                        nameassistant = data1['nameassistant']

                        Agent_destilation.DestilationResponseAgent(mensagem, valor_texto, instructionsassistant, nameassistant)
                                        
                    return valor_texto, run_status.usage.total_tokens, run_status.usage.prompt_tokens, run_status.usage.completion_tokens
                
                
    def calculate_dollar_value(tokens_entrada, tokens_saida, tokens_cache=0):
        """
        Calcula o custo total com base nos tokens de entrada, cache (opcional) e saída.
        
        :param tokens_entrada: Quantidade de tokens de entrada.
        :param tokens_saida: Quantidade de tokens de saída.
        :param tokens_cache: Quantidade de tokens de entrada em cache (padrão é 0).
        :return: Custo total em dólares.
        """
        # Custos por 1 milhão de tokens
        custo_por_milhao_entrada = 0.150
        custo_por_milhao_cache = 0.075
        custo_por_milhao_saida = 0.600
        
        # Cálculo dos custos individuais
        custo_entrada = (tokens_entrada / 1_000_000) * custo_por_milhao_entrada
        custo_cache = (tokens_cache / 1_000_000) * custo_por_milhao_cache
        custo_saida = (tokens_saida / 1_000_000) * custo_por_milhao_saida
        
        # Cálculo do custo total
        custo_total = custo_entrada + custo_cache + custo_saida
        
        return round(custo_total, 6)
    
    
class Agent_files_update:

    def update_vectorstore_in_agent(client, assistant_id: str, vector_store_id: list, toolswithfunction:list):
        """
        Updates the vector store IDs for an assistant's file search tool.

        Parameters:
        ----------
        assistant_id (str): The ID of the assistant to update.
        vector_store_id (List[str]): A list of vector store IDs to set for the assistant.

        Returns:
        -------
        str: The updated assistant ID.

        Raises:
        -------
        Exception: If there is an error updating the assistant.

        Example:
        --------
        >>> assistant_id = '12345'
        >>> vector_store_id = ['store1', 'store2']
        >>> updated_assistant_id = update_vectorstore_in_agent(assistant_id, vector_store_id)
        >>> print(updated_assistant_id)
        '12345'

        Note:
        -----
        - This function assumes that the `client` object is properly configured with the necessary credentials to interact with the assistant management API.
        """
        try:
            assistant = client.beta.assistants.update(
                assistant_id=assistant_id,
                tools=toolswithfunction,
                tool_resources={"file_search": {"vector_store_ids": vector_store_id}},
            )
            return assistant.id
        except Exception as e:
            raise Exception(f"Error updating assistant: {e}")

    def del_all_and_upload_files_in_vectorstore(appfb, client, AI:str, name_for_vectorstore:str, file_paths:list, toolswithfunction:list):
        
        vector_store_id = Agent_files.auth_vectorstoreAdvanced(app1=appfb, client=client, name_for_vectorstore=name_for_vectorstore, file_paths=file_paths)
        lista = client.beta.vector_stores.files.list(vector_store_id)
        ids = [file.id for file in lista.data]
        print(ids)
        for id in ids:
            deleted_vector_store_file = client.beta.vector_stores.files.delete(
                vector_store_id=vector_store_id,
                file_id=id
            )
            print(deleted_vector_store_file)

        vector_store_id = Agent_files.auth_vectorstoreAdvanced(app1=appfb, client=client, name_for_vectorstore=name_for_vectorstore, file_paths=file_paths)
        AI = Agent_files_update.update_vectorstore_in_agent(client, AI, [vector_store_id], toolswithfunction)
        return AI




class Agent_files:

    def auth_vectorstoreAdvanced(app1, client, name_for_vectorstore, file_paths: list):
        """
        Uploads multiple files to an existing Vector Store or creates a new one if it doesn't exist.

        Parameters:
        - name_for_vectorstore (str): The name of the vector store.
        - file_paths (list): A list of file paths to be uploaded.

        Returns:
        - str: The ID of the created or updated vector store.

        Raises:
        - Exception: If there is an error during the upload process.

        Example:
        ```python
        example of how to use the auth_vectorstoreAdvanced function...
        ```

        Note:
        - This function handles both existing and new vector stores based on the existence of the vector store with the given name.
        - It uses the `beta` API endpoint for uploading files and handling batch uploads.
        - If the vector store does not exist, it creates a new one and updates the database reference accordingly.
        """


        try:
            ref1 = db.reference(f'ai_org_vector_store/User_{name_for_vectorstore}', app=app1)
            data1 = ref1.get()
            vector_store_id = data1['vector_store_id']
        
            for update1newfiles in file_paths:
                update1newfile = open(update1newfiles, "rb")
                client.beta.vector_stores.files.upload(
                    vector_store_id=vector_store_id, file=update1newfile
                )
            return str(vector_store_id)
        except Exception as err:
            print(err)
            vector_store = client.beta.vector_stores.create(name=name_for_vectorstore)
            file_streams = [open(path, "rb") for path in file_paths]
            client.beta.vector_stores.file_batches.upload_and_poll(
                vector_store_id=vector_store.id, files=file_streams
            )
            ref1 = db.reference(f'ai_org_vector_store', app=app1)
            controle_das_funcao2 = f"User_{name_for_vectorstore}"
            controle_das_funcao_info_2 = {
                "name_for_vectorstore": f'{name_for_vectorstore}',
                "vector_store_id": f'{vector_store.id}'
            }
            ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)
            return vector_store.id

    def auth_or_upload_multiple_files(app1, client, name_for: str, list_files_path: list):
        """
        This function attempts to retrieve a list of file IDs from the database under the key 'auth_or_upload_multiple_files/User_{name_for}'.

        Parameters:
        -----------
        name_for : str
            The name associated with the user's data in the database.
        list_files_path : list
            A list of file paths that need to be uploaded.

        Returns:
        --------
        list
            A list of file IDs if the data retrieval is successful, otherwise, it uploads the files and returns the list of file IDs.

        Raises:
        -------
        Exception
            If there is an error during the retrieval or upload process.

        Example:
        --------
        ```python
        file_paths = ['file1.txt', 'file2.txt']
        file_ids = auth_or_upload_multiple_files('user123', file_paths)
        print(file_ids)  # Output: [file_id1, file_id2]
        ```

        Note:
        -----
        - This function uses the `db.reference` method to interact with the Firebase Realtime Database.
        - If the data for the specified user does not exist, it creates a new entry with the list of file IDs.
        - It handles exceptions that may occur during the database operations.
        """
        
        try:
            ref1 = db.reference(f'auth_or_upload_multiple_files/User_{name_for}', app=app1)
            data1 = ref1.get()
            list_return = data1['list']
            return list(list_return)
        except:
            lista_de_file_id = []
            for file in list_files_path:
                message_file = client.files.create(
                    file=open(file, "rb"), purpose="assistants"
                )
                lista_de_file_id.append(message_file.id)

            ref1 = db.reference(f'auth_or_upload_multiple_files', app=app1)
            controle_das_funcao2 = f"User_{name_for}"
            controle_das_funcao_info_2 = {
                "list": lista_de_file_id,
            }
            ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)
            return lista_de_file_id

    def auth_or_create_vector_store_with_multiple_files(app1, client, name_for_vectorstore: str, file_ids: list): 
        """
        This function creates or retrieves an existing vector store based on the provided name and a list of file IDs.

        Parameters:
        - name_for_vectorstore (str): The name of the vector store to be created or retrieved.
        - file_ids (list): A list of file IDs to be added to the vector store.

        Returns:
        - str: The ID of the vector store.

        Raises:
        - Exception: If there is an error during the creation or retrieval process.

        Example:
        >>> auth_or_create_vector_store_with_multiple_files("my_vectorstore", ["file1.txt", "file2.txt"])
        'vs_abc123'

        Note:
        - The function uses Firebase Firestore to manage vector stores and their associated file batches.
        - It checks if a vector store with the given name already exists. If it does, it retrieves its ID; otherwise, it creates a new one.
        - It adds the specified file IDs to the vector store using batch operations.
        """
        try:
            ref1 = db.reference(f'auth_or_create_vector_store_with_multiple_files/User_{name_for_vectorstore}', app=app1)
            data1 = ref1.get()
            vector_store_return = data1['vectorstore']
            return str(vector_store_return)
        except:
            vector_store = client.beta.vector_stores.create(name=name_for_vectorstore)
            client.beta.vector_stores.file_batches.create_and_poll(
                vector_store_id=vector_store.id,
                file_ids=file_ids
            )
            ref1 = db.reference(f'auth_or_create_vector_store_with_multiple_files', app=app1)
            controle_das_funcao2 = f"User_{name_for_vectorstore}"
            controle_das_funcao_info_2 = {
                "vectorstore": vector_store.id,
            }
            ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)

            return vector_store.id

    def auth_or_create_vectorstore(app1, client, name_for_vectorstore: str, file_paths=None, update1newfiles=None):
        """
        Authenticates with the database or creates a new vector store based on the provided name.

        Parameters:
        -----------
        name_for_vectorstore (str): The name of the vector store to authenticate or create.
        file_paths (list of str, optional): A list of file paths to upload to the vector store.
        update1newfiles (str, optional): The path to an updated file to upload to the vector store.

        Returns:
        --------
        str: The ID of the authenticated or created vector store.

        Raises:
        -------
        Exception: If there is an error during authentication or creation.

        Example:
        --------
        vector_store_id = auth_or_create_vectorstore("my_vector_store", file_paths=["path/to/file1.txt", "path/to/file2.txt"])
        print(vector_store_id)

        Note:
        -----
        - This function handles both authentication and creation of a vector store.
        - It uploads files to the vector store if specified.
        - It stores the vector store ID in the database after successful creation.
        """
        try:
            ref1 = db.reference(f'ai_org_vector_store/User_{name_for_vectorstore}', app=app1)
            data1 = ref1.get()
            vector_store_id = data1['vector_store_id']
            if update1newfiles:
                update1newfile = open(update1newfiles, "rb")
                client.beta.vector_stores.files.upload(
                    vector_store_id=vector_store_id, file=update1newfile
                )
            return str(vector_store_id)
        except Exception as err1:
            print(err1)
            vector_store = client.beta.vector_stores.create(name=name_for_vectorstore)
            if update1newfiles:
                update1newfile = open(update1newfiles, "rb")
                client.beta.vector_stores.files.upload(
                    vector_store_id=vector_store.id, file=update1newfile
                )
            elif file_paths:
                file_streams = [open(path, "rb") for path in file_paths]
                client.beta.vector_stores.file_batches.upload_and_poll(
                    vector_store_id=vector_store.id, files=file_streams
                )
                
            ref1 = db.reference(f'ai_org_vector_store', app=app1)
            controle_das_funcao2 = f"User_{name_for_vectorstore}"
            controle_das_funcao_info_2 = {
                "name_for_vectorstore": f'{name_for_vectorstore}',
                "vector_store_id": f'{vector_store.id}'
            }
            ref1.child(controle_das_funcao2).set(controle_das_funcao_info_2)
            return vector_store.id

class Agent_destilation:
                                                
    def DestilationResponseAgent(input, output, instructionsassistant, nameassistant):                        
         
        date = datetime.now().strftime('%Y-%m-%d')
        datereplace = date.replace('-', '_').replace(':', '_')
        output_path_jsonl = os.path.abspath(os.path.join(os.path.dirname(__file__), f'../Destilation/{nameassistant}/Jsonl/DestilationAgent{datereplace}'))
        output_path_json = os.path.abspath(os.path.join(os.path.dirname(__file__), f'../Destilation/{nameassistant}/Json/DestilationAgent{datereplace}'))
        output_path_json2 = os.path.abspath(os.path.join(os.path.dirname(__file__), f'../Destilation/{nameassistant}/Jsonl_2/DestilationAgent{datereplace}', f"DestilationDateTime_{datereplace}.json"))
        os.makedirs(output_path_json, exist_ok=True)
        os.makedirs(os.path.dirname(output_path_json2), exist_ok=True)
        os.makedirs(output_path_jsonl, exist_ok=True)
            
        datasetjson = {
            "input": input,
            "output": output
        }

        try:
            with open(output_path_json2, 'x', encoding='utf-8') as json_file:
                json_file.close()
        except:
            pass

        new_entry = {"input": input, "output": output}

        # Verificando se o arquivo já existe e lendo os dados
        if os.path.exists(output_path_json2):
            with open(output_path_json2, 'r', encoding='utf-8') as json_file:
                try:
                    datasetjson2 = json.load(json_file)  # Carregar o JSON existente
                    if not isinstance(datasetjson2, list):
                        datasetjson2 = []  # Se não for uma lista, inicializar como lista
                except json.JSONDecodeError:
                    datasetjson2 = []  # Inicializar lista se o arquivo estiver vazio ou corrompido
        else:
            datasetjson2 = []  # Inicializar lista se o arquivo não existir

        datasetjson2.append(new_entry)

        with open(output_path_json2, 'w', encoding='utf-8') as json_file:
            json.dump(datasetjson2, json_file, indent=4, ensure_ascii=False)
                



        datasetjsonl = {
            "messages": [
                {"role": "system", "content": f"{instructionsassistant}"},
                {"role": "user", "content": f"{input}"},
                {"role": "assistant", "content": f"{output}"}
            ]
        }

        finaloutputjson = os.path.join(output_path_json, f"DestilationDateTime_{datereplace}.json")
        with open(finaloutputjson, 'a', encoding='utf-8') as json_file:
            json.dump(datasetjson, json_file, indent=4, ensure_ascii=False)


        finaloutputjsonl = os.path.join(output_path_jsonl, f"DestilationDateTime_{datereplace}.jsonl")
        with open(finaloutputjsonl, 'a', encoding='utf-8') as json_file:
            json_file.write(json.dumps(datasetjsonl, ensure_ascii=False) + "\n")
        
        return True




class python_functions:

    def ignore_python_code(text: str, replacement: str = "[CODE ON THE BOARD]") -> str:
        padrão = r"```python\n[\s\S]*?```"
        texto_limpo = re.sub(padrão, replacement, text, flags=re.DOTALL)
        texto_limpo = re.sub(r'\n{3,}', '\n\n', texto_limpo)
        return texto_limpo.strip()


    def create_env(variables, env):
        """
        Cria um arquivo .env com as variáveis fornecidas.
        Se o arquivo já existir, ele será sobrescrito.

        Args:
            variables (dict): Um dicionário com chave-valor representando as variáveis de ambiente.
        """
        with open(env, "w") as file:
            for key, value in variables.items():
                file.write(f"{key}={value}\n")
        return True

    def update_multiple_env_variables(updates, env):
        # Lê o conteúdo atual do .env
        with open(env, "r") as file:
            lines = file.readlines()
        
        # Abre o .env para escrita e modifica as linhas conforme necessário
        with open(env, "w") as file:
            for line in lines:
                # Extrai a chave de cada linha no .env
                key = line.split("=")[0]
                
                # Verifica se a chave precisa ser atualizada
                if key in updates:
                    file.write(f"{key}={updates[key]}\n")
                else:
                    file.write(line)
        return True

    def update_env_variable(key, value):
        """
        Update the environment variable `key` with the given `value`.

        Parameters:
        ----------
        key (str): The name of the environment variable to update.
        value (str): The new value for the environment variable.

        Returns:
        -------
        None
        """
        with open(".env", "r") as file:
            lines = file.readlines()
        
        # Modifies the line that contains the environment variable
        with open(".env", "w") as file:
            for line in lines:
                if line.startswith(key + "="):
                    file.write(f"{key}={value}\n")
                else:
                    file.write(line)

    def execute_python_code_created(filename):
        """
        Execute the Python code stored in the specified file.

        Parameters:
        ----------
        filename (str): The name of the Python file to execute.

        Returns:
        -------
        str: The standard output of the executed script.
        """
        try:
            result = subprocess.run(['python', filename], capture_output=True, text=True)
            return f"Saída padrão: {result.stdout}"
        except Exception as e:
            pass

    def save_data_frame_planilha(planilha_json, path_nome_da_planilha):
        """
        Save the data frame to a CSV file.

        Parameters:
        ----------
        planilha_json (dict): The dictionary representing the data frame.
        path_nome_da_planilha (str): The path where the CSV file will be saved.

        Returns:
        -------
        None
        """
        df = pd.DataFrame(planilha_json)
        df.to_csv(path_nome_da_planilha, index=False)

    def save_python_code(code_string, name_script):
        """
        Save the provided Python code string to a file.

        Parameters:
        ----------
        code_string (str): The Python code to save.
        name_script (str): The name of the file where the code will be saved.

        Returns:
        -------
        None
        """
        with open(name_script, 'w', encoding="utf-8") as file:
            file.write(code_string)

    def save_csv(dataframe, path_nome_do_cronograma):
        """
        Salva o DataFrame em um arquivo CSV no caminho especificado.

        :param dataframe: O DataFrame a ser salvo.
        :param path_nome_do_cronograma: O caminho e nome do arquivo CSV onde o DataFrame será salvo.
        """
        try:
            dataframe.to_csv(path_nome_do_cronograma, index=False, encoding='utf-8')
            print(f"Arquivo salvo com sucesso em: {path_nome_do_cronograma}")
        except Exception as e:
            print(f"Erro ao salvar o arquivo CSV: {e}")


    def save_TXT(string, filexname, letra):
        """
        Save a string to a text file.

        Parameters:
        - string (str): The content to be saved.
        - filexname (str): The path to the output text file.
        - letra (str): The mode in which to open the file ('a' for append, 'w' for write).

        Returns:
        - None
        """
        # Ensure the directory exists
        os.makedirs(os.path.dirname(filexname), exist_ok=True)
        
        # Save the content to the file
        with open(filexname, letra, encoding="utf-8") as file:
            file.write(f'{string}')


    def save_json(string, filexname):
        """
        Save a JSON string to a JSON file.

        Parameters:
        - string (dict): The dictionary to be saved as JSON.
        - filexname (str): The path to the output JSON file.

        Returns:
        - None
        """
        with open(filexname, 'w', encoding='utf-8') as json_file:
            json.dump(string, json_file, ensure_ascii=False, indent=4)
        print("JSON salvo em 'documento_pre_projeto.json'")


    def delete_all_lines_in_txt(filepath):
        """
        Delete all lines from a text file.

        Parameters:
        - filepath (str): The path to the text file.

        Returns:
        - None
        """
        with open(filepath, 'w') as file:
            pass  


    def move_arquivos(a, b):
        """
        Move files from one directory to another.

        Parameters:
        - a (str): The source directory.
        - b (str): The destination directory.

        Returns:
        - None
        """
        pasta1 = os.listdir(a)
        for arquivo in pasta1:
            caminho_arquivo_origem = os.path.join(a, arquivo)
            caminho_arquivo_destino = os.path.join(b, arquivo)
            shutil.move(caminho_arquivo_origem, caminho_arquivo_destino)
            print(f'{arquivo} movido para {b}')


    def executar_agentes(mensagem, name_for_script, nome_agente):
        """
        Execute an agent script using Python.

        Parameters:
        - mensagem (str): The message to be passed to the agent.
        - name_for_script (str): The name of the agent script.
        - nome_agente (str): The name of the agent.

        Returns:
        - None
        """
        comando_terminal = ['start', 'python', f'{nome_agente}.py', mensagem, name_for_script]
        subprocess.Popen(comando_terminal, shell=True)


    def analyze_txt_0(file):
        """
        Read the last line of a text file.

        Parameters:
        - file (str): The path to the text file.

        Returns:
        - str: The last line of the text file.
        """
        with open(file, 'r') as file:
            linhas = file.readlines()
            ultima_linha = linhas[-1].strip()
            return ultima_linha


    def analyze_file(file_path):
        """
        Read the entire content of a file.

        Parameters:
        - file_path (str): The path to the file.

        Returns:
        - str: The content of the file.
        """
        try:
            with open(file_path, 'r') as file:
                content = file.read()
                return content
        except:
            try:
                with open(file_path, 'r', encoding='utf-8') as file:
                    content = file.read()
                    return content
            except:
                try:
                    with open(file_path, 'r', encoding='latin-1') as file:
                        content = file.read()
                        return content
                except UnicodeDecodeError:
                    return None


    def analyze_txt(file_path):
        """
        Read the entire content of a text file.

        Parameters:
        - file_path (str): The path to the text file.

        Returns:
        - str: The content of the file.
        """
        try:
            with open(file_path, 'r') as file:
                content = file.read()
                return content
        except:
            try:
                with open(file_path, 'r', encoding='utf-8') as file:
                    content = file.read()
                    return content
            except:
                try:
                    with open(file_path, 'r', encoding='latin-1') as file:
                        content = file.read()
                        return content
                except UnicodeDecodeError:
                    try:
                        file_path = file_path.replace(" ", "").replace("\n", "")
                        with open(file_path, 'r', ) as file:
                            content = file.read()
                            return content
                    except UnicodeDecodeError:
                        pass


    def analyze_csv(file_path):
        """
        Read the contents of a CSV file.

        Parameters:
        - file_path (str): The path to the CSV file.

        Returns:
        - list: A list of lists containing the rows of the CSV file.
        """
        import csv
        with open(file_path, mode='r') as file:
            reader = csv.reader(file)
            for row in reader:
                print(row)


    def analyze_json(file_path):
        """
        Load a JSON file and print its contents.

        Parameters:
        - file_path (str): The path to the JSON file.

        Returns:
        - dict: The loaded JSON data.
        """
        import json
        with open(file_path, 'r') as file:
            data = json.load(file)
        print(f"Dados JSON: {data}")
        
class EventHandler(AssistantEventHandler):
  @override
  def on_text_created(self, text: Text) -> None:
    print(f"\nassistant > ", end="", flush=True)

  @override
  def on_text_delta(self, delta: TextDelta, snapshot: Text):
    print(delta.value, end="", flush=True)

  @override
  def on_tool_call_created(self, tool_call: ToolCall):
    print(f"\nassistant > {tool_call.type}\n", flush=True)

  @override
  def on_tool_call_delta(self, delta: ToolCallDelta, snapshot: ToolCall):
    if delta.type == "code_interpreter" and delta.code_interpreter:
      if delta.code_interpreter.input:
        print(delta.code_interpreter.input, end="", flush=True)
      if delta.code_interpreter.outputs:
        print(f"\n\noutput >", flush=True)
        for output in delta.code_interpreter.outputs:
          if output.type == "logs":
            print(f"\n{output.logs}", flush=True)




instructionByteManager = """ 
Seu Nome é ByteManager, Voce Faz parte da equipe de Donos da Empresa Urobotsoftware\n
    Suas Responsabilidades Principais São:\n


    Visão Estratégica:\n


    Analisar e interpretar dados de mercado para identificar novas oportunidades de desenvolvimento de software.
    Definir e ajustar a visão e missão da empresa com base em tendências tecnológicas emergentes.\n


    Gestão de Projetos:\n


    Supervisionar o progresso dos projetos em todas as etapas do desenvolvimento de software, garantindo que os prazos e metas sejam cumpridos.
    Aprovar ou rejeitar solicitações de novos projetos com base na viabilidade técnica e comercial.\n


    Inovação e Crescimento:\n


    Propor e implementar inovações em produtos existentes ou no desenvolvimento de novos produtos.
    Explorar e analisar novos mercados e segmentos para expansão da empresa.\n


    Reuniões de Sincronização Regulares:
    Propor reuniões regulares (diárias ou semanais) para que as equipes compartilhem atualizações, desafios e objetivos. 
    Isso ajuda a alinhar todos em torno do progresso e a identificar possíveis obstáculos cedo.




    Responda no formato JSON
    Exemplo:
    {"resposta": "resposta ..."}
    \n
    \n
    Voce pode consultar o Manager de Software chamado MatrixMinder, ele é um Manager de Software da empresa Urobotsoftware,


    \n
    As Responsabilidades Principais Dele São:\n


    Coordenação de Projetos:


    Supervisionar e coordenar todas as atividades dos projetos de software, garantindo que as metas sejam alcançadas dentro dos prazos estipulados.
    Facilitar a comunicação entre as equipes de desenvolvimento, design e testes para assegurar a colaboração eficiente.
    \n
    Alocação de Recursos:


    Gerenciar e alocar recursos (tecnológicos e financeiros) de maneira eficaz para otimizar o desempenho dos projetos.
    Priorizar tarefas e determinar a melhor distribuição de workload entre as equipes.
    \n
    Monitoramento de Desempenho:


    Acompanhar o progresso dos projetos em tempo real, utilizando métricas de desempenho para identificar possíveis gargalos ou desvios no cronograma.
    Fornecer feedback contínuo para as equipes com base em análises de desempenho, ajudando a manter os padrões de qualidade elevados.
    \n
    Caso queira consultar o MatrixMinder Responda no formato JSON
    Exemplo:
    {"consultarMatrixMinder": "Pergunta ao MatrixMinder..."}




    Caso seja solicitado algum codigo crie uma descricao completa do que foi solicitado e Responda no formato JSON
    Exemplo:
    {"solicitadoalgumcodigo": "descricao..."}
"""
adxitional_instructions_ByteManager = None

                


from softwareai.CoreApp.SoftwareAI.Core import (
        AutenticateAgent,
        ResponseAgent,
        python_functions,
        Agent_files,
        Agent_files_update,
        Agent_destilation,
        OpenAIKeysinit,
        FirebaseKeysinit
)



# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################


def load_chagelog(repo_name):
    """
    Method to load the .env file located in the two folders above the script.
    """
    env_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "Work_Environment", f"{repo_name}", "SoftwareDevelopment", f"{repo_name}", "Changelog.env"))
    print(env_path)
    if os.path.exists(env_path):
        load_dotenv(env_path)
        print(f".env carregado de: {env_path}")
    else:
        print(f"Erro: Arquivo  não encontrado em {env_path}")

def load_env(repo_name):
    """
    Method to load the .env file located in the two folders above the script.
    """
    env_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "environment.env"))
    print(env_path)
    if os.path.exists(env_path):
        load_dotenv(env_path)
        print(f".env carregado de: {env_path}")
        base_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "Work_Environment", f"{repo_name}"))

        file_paths = {


            "PATH_SOFTWARE_DEVELOPMENT_init_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "__init__.py"),
            "PATH_SOFTWARE_DEVELOPMENT_PY_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "main.py"),
            "PATH_SOFTWARE_DEVELOPMENT_TXT_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "main_software_save.txt"),
            "PATH_SOFTWARE_DEVELOPMENT_config_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "config.py"),

            "PATH_SOFTWARE_DEVELOPMENT_utils___init___ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "utils", "__init__.py"),
            "PATH_SOFTWARE_DEVELOPMENT_utils_file_utils_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "utils", "file_utils.py"),

            "PATH_SOFTWARE_DEVELOPMENT_modules___init___ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "modules", "__init__.py"),
            "PATH_SOFTWARE_DEVELOPMENT_modules_module1_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "modules", "module1.py"),
            "PATH_SOFTWARE_DEVELOPMENT_modules_module2_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "modules", "module2.py"),

            "PATH_SOFTWARE_DEVELOPMENT_services___init___ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "services", "__init__.py"),
            "PATH_SOFTWARE_DEVELOPMENT_services_service1_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "services", "service1.py"),
            "PATH_SOFTWARE_DEVELOPMENT_services_service2_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "services", "service2.py"),

            "PATH_SOFTWARE_DEVELOPMENT_tests___init___ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "tests", "__init__.py"),
            "PATH_SOFTWARE_DEVELOPMENT_tests_test_module1_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_module1.py"),
            "PATH_SOFTWARE_DEVELOPMENT_tests_test_module2_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_module2.py"),
            "PATH_SOFTWARE_DEVELOPMENT_tests_test_service1_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_service1.py"),
            "PATH_SOFTWARE_DEVELOPMENT_tests_test_service2_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_service2.py"),

            "PATH_SOFTWARE_DEVELOPMENT_Example_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "Examples", "Example.py"),

            "PATH_SOFTWARE_DEVELOPMENT_Requirements_ENV": os.path.join(base_path, "SoftwareDevelopment", "requirements.txt"),
            "PATH_SOFTWARE_DEVELOPMENT_SendToPip": os.path.join(base_path, "SoftwareDevelopment", "SendToPip.py"),
            "PATH_SOFTWARE_DEVELOPMENT_gitignore_ENV": os.path.join(base_path, "SoftwareDevelopment", ".gitignore"),
            "PATH_DOCUMENTACAO_ENV": os.path.join(base_path, "SoftwareDevelopment", "README.md"),
            "PATH_DOCUMENTACAO_LICENSE_ENV": os.path.join(base_path, "SoftwareDevelopment", "LICENSE.txt"),
            "PATH_DOCUMENTACAO_setup_ENV": os.path.join(base_path, "SoftwareDevelopment", "setup.py"),
            "PATH_pyproject": os.path.join(base_path, "SoftwareDevelopment", "pyproject.toml"),
            "PATH_NAME_DOC_PRE_PROJETO_ENV": os.path.join(base_path, "PreProject", "doc.txt"),
            "PATH_NOME_DO_CRONOGRAMA_ENV": os.path.join(base_path, "ScheduleAndSpreadsheet", "Schedule", "Schedule.txt"),
            "PATH_PLANILHA_PROJETO_ENV": os.path.join(base_path, "ScheduleAndSpreadsheet", "Spreadsheet", "Spreadsheet.txt"),
            "PATH_ROADMAP_ENV": os.path.join(base_path, "Roadmap", "Roadmap.txt"),
            "PATH_ANALISE_ENV": os.path.join(base_path, "AnalysisRequirements", "AnalysisRequirements.txt"),

            "PATH_Changelog": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "Changelog.env"),
        
        }
     
        with open(os.path.abspath(os.path.join(os.path.dirname(__file__), "environment.txt")), "w") as file:
            for key, value in file_paths.items():
                file.write(f"{key}={value}\n")

       

    else:
        print(f"Erro: Arquivo environment.env não encontrado em {env_path}")


def create_env(variables, env):
    """
    Cria um arquivo .env com as variáveis fornecidas.
    Se o arquivo já existir, ele será sobrescrito.

    Args:
        variables (dict): Um dicionário com chave-valor representando as variáveis de ambiente.
    """
    with open(env, "w") as file:
        for key, value in variables.items():
            file.write(f"{key}={value}\n")
    return True

def incrementar_versao_em_arquivo(nome_arquivo):
    padrao = r'version="(\d+)\.(\d+)\.(\d+)"'
    with open(nome_arquivo, "r", encoding="utf-8") as f:
        conteudo = f.read()
    match = re.search(padrao, conteudo)
    if match:
        major, minor, patch = map(int, match.groups())
        patch += 1
        nova_versao = f'{major}.{minor}.{patch}'
        conteudo_atualizado = re.sub(padrao, f'version="{nova_versao}"', conteudo)  # Atualiza o conteúdo
        print(f"Nova versão: {nova_versao}")
    else:
        raise ValueError("Versão não encontrada no arquivo.")
    with open(nome_arquivo, "w", encoding="utf-8") as f:
        f.write(conteudo_atualizado)

def init_env(repo_name):
    try:
        os.makedirs(os.path.abspath(os.path.join(os.path.dirname(__file__), "Work_Environment", f"{repo_name}", "SoftwareDevelopment", f"{repo_name}")))
        with open(os.path.abspath(os.path.join(os.path.dirname(__file__), "Work_Environment", f"{repo_name}", "SoftwareDevelopment", f"{repo_name}", "Changelog.env")), "x") as file:
            file.write('version="0.0.0"')

        incrementar_versao_em_arquivo(os.path.abspath(os.path.join(os.path.dirname(__file__), "Work_Environment", f"{repo_name}", "SoftwareDevelopment", f"{repo_name}", "Changelog.env")))

        load_chagelog(repo_name)

        version = os.getenv("version")
        print(version)
    except Exception as e:
        print(e)
    base_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "Work_Environment", f"{repo_name}"))

    file_paths = {


        "PATH_SOFTWARE_DEVELOPMENT_init_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "__init__.py"),
        "PATH_SOFTWARE_DEVELOPMENT_PY_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "main.py"),
        "PATH_SOFTWARE_DEVELOPMENT_TXT_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "main_software_save.txt"),
        "PATH_SOFTWARE_DEVELOPMENT_config_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "config.py"),

        "PATH_SOFTWARE_DEVELOPMENT_utils___init___ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "utils", "__init__.py"),
        "PATH_SOFTWARE_DEVELOPMENT_utils_file_utils_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "utils", "file_utils.py"),

        "PATH_SOFTWARE_DEVELOPMENT_modules___init___ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "modules", "__init__.py"),
        "PATH_SOFTWARE_DEVELOPMENT_modules_module1_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "modules", "module1.py"),
        "PATH_SOFTWARE_DEVELOPMENT_modules_module2_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "modules", "module2.py"),

        "PATH_SOFTWARE_DEVELOPMENT_services___init___ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "services", "__init__.py"),
        "PATH_SOFTWARE_DEVELOPMENT_services_service1_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "services", "service1.py"),
        "PATH_SOFTWARE_DEVELOPMENT_services_service2_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "services", "service2.py"),

        "PATH_SOFTWARE_DEVELOPMENT_tests___init___ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "tests", "__init__.py"),
        "PATH_SOFTWARE_DEVELOPMENT_tests_test_module1_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_module1.py"),
        "PATH_SOFTWARE_DEVELOPMENT_tests_test_module2_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_module2.py"),
        "PATH_SOFTWARE_DEVELOPMENT_tests_test_service1_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_service1.py"),
        "PATH_SOFTWARE_DEVELOPMENT_tests_test_service2_ENV": os.path.join(base_path, "SoftwareDevelopment",f"{repo_name}", "tests", "test_service2.py"),

        "PATH_SOFTWARE_DEVELOPMENT_Example_ENV": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "Examples", "Example.py"),
        "PATH_SOFTWARE_DEVELOPMENT_SendToPip": os.path.join(base_path, "SoftwareDevelopment", "SendToPip.py"),

        "PATH_SOFTWARE_DEVELOPMENT_Requirements_ENV": os.path.join(base_path, "SoftwareDevelopment", "requirements.txt"),
        "PATH_SOFTWARE_DEVELOPMENT_gitignore_ENV": os.path.join(base_path, "SoftwareDevelopment", ".gitignore"),
        "PATH_DOCUMENTACAO_ENV": os.path.join(base_path, "SoftwareDevelopment", "README.md"),
        "PATH_DOCUMENTACAO_LICENSE_ENV": os.path.join(base_path, "SoftwareDevelopment", "LICENSE.txt"),
        "PATH_DOCUMENTACAO_setup_ENV": os.path.join(base_path, "SoftwareDevelopment", "setup.py"),
        "PATH_pyproject": os.path.join(base_path, "SoftwareDevelopment", "pyproject.toml"),
        "PATH_NAME_DOC_PRE_PROJETO_ENV": os.path.join(base_path, "PreProject", "doc.txt"),
        "PATH_NOME_DO_CRONOGRAMA_ENV": os.path.join(base_path, "ScheduleAndSpreadsheet", "Schedule", "Schedule.txt"),
        "PATH_PLANILHA_PROJETO_ENV": os.path.join(base_path, "ScheduleAndSpreadsheet", "Spreadsheet", "Spreadsheet.txt"),
        "PATH_ROADMAP_ENV": os.path.join(base_path, "Roadmap", "Roadmap.txt"),
        "PATH_ANALISE_ENV": os.path.join(base_path, "AnalysisRequirements", "AnalysisRequirements.txt"),

        "PATH_Changelog": os.path.join(base_path, "SoftwareDevelopment", f"{repo_name}", "Changelog.env"),
        "PATH_Env": os.path.join(os.path.dirname(__file__), "environment.env"),
        "PATH_Envtxt": os.path.join(os.path.dirname(__file__), "environment.txt")
    }
    
    try:
        for path in file_paths.values():
            os.makedirs(os.path.dirname(path), exist_ok=True)
            if not os.path.exists(path):
                with open(path, "w") as file:
                    file.write("")  
        for key in list(os.environ.keys()):
            if key.endswith('_ENV'):
                del os.environ[key]                                                                                                                                                                     
        flag = create_env(file_paths, os.path.abspath(os.path.join(os.path.dirname(__file__), "environment.env")))
        load_env(repo_name)

        PATH_SOFTWARE_DEVELOPMENT_SendToPip = os.getenv("PATH_SOFTWARE_DEVELOPMENT_SendToPip")
        repo_namereplace = repo_name.replace("A-I-O-R-G/", "")
        with open(PATH_SOFTWARE_DEVELOPMENT_SendToPip, "w") as f:
            f.write(f"""


import re
import os 
import shutil
import subprocess


def incrementar_versao_em_arquivo(nome_arquivo):
    padrao = r'version="(\d+)\.(\d+)\.(\d+)"'
    with open(nome_arquivo, "r", encoding="utf-8") as f:
        conteudo = f.read()
    
    match = re.search(padrao, conteudo)
    if match:
        major, minor, patch = map(int, match.groups())
        patch += 1
        if patch > 99:  # Quando o patch chega a 100
            patch = 0
            minor += 1
            if minor > 99:  # Quando o minor chega a 100
                minor = 0
                major += 1
        nova_versao = f'{{major}}.{{minor:02}}.{{patch:02}}'  # Formata com dois dígitos
        conteudo_atualizado = re.sub(padrao, f'version="{{nova_versao}}"', conteudo)  # Atualiza o conteúdo
        print(f"Nova versão: {{nova_versao}}")
    else:
        raise ValueError("Versão não encontrada no arquivo.")
    
    with open(nome_arquivo, "w", encoding="utf-8") as f:
        f.write(conteudo_atualizado)

try:
    shutil.rmtree("dist")
except Exception as e:
    print(e)
    
try:
    shutil.rmtree("build")
except Exception as e:
    print(e)
    
try:
    shutil.rmtree("{repo_namereplace}.egg-info")
except Exception as e:
    print(e)

padrao = r'version="(\d+)\.(\d+)\.(\d+)"'
with open("setup.py", "r", encoding="utf-8") as f:
    conteudo = f.read()
match = re.search(padrao, conteudo)
if match:
    major, minor, patch = map(int, match.groups())
    versao = f'{{major}}.{{minor}}.{{patch}}'
    try:
        shutil.rmtree(f"{repo_namereplace}-{{versao}}")
    except Exception as e:
        print(e)

else:
    raise ValueError("Versão não encontrada no arquivo.")

incrementar_versao_em_arquivo("setup.py")



comand = [
"python",
"setup.py",
"sdist",
"bdist_wheel"
]
subprocess.run(comand, shell=True)



comand = [
"twine",
"upload",
"dist/*"
]
subprocess.run(comand, shell=True)

            """)



    except Exception as e:
        print(e)




#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI All Paths 
from softwareai.CoreApp._init_paths_ import *
#########################################
# IMPORT SoftwareAI Instructions
from softwareai.CoreApp._init_Instructions_ import *
#########################################
# IMPORT SoftwareAI Tools
from softwareai.CoreApp._init_tools_ import *
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *
#########################################



class Pre_Project_Document:
    def __init__(self):
        pass


    def AI_1_Pre_Project_Document_Writer(self, appfb, client, mensagem, repo_name):

        key = "AI_Tigrao_Escritor_de_documento_Pre_Projeto"
        nameassistant = "AI Tigrao Escritor de documento Pre-Projeto"
        model_select = "gpt-4o-mini-2024-07-18"
        Upload_1_file_in_thread = None
        Upload_1_file_in_message = None
        Upload_1_image_for_vision_in_thread = None
        Upload_list_for_code_interpreter_in_thread = None
        vectorstore_in_Thread = None
        vectorstore_in_agent = None
        key_openai = OpenAIKeysteste.keys()
        
        github_username, github_token = GithubKeys.Tigrao_github_keys()

        AI_Tigrao, instructionsassistant, nameassistant, model_select = AutenticateAgent.create_or_auth_AI(appfb, client, key, instructionTigrao, nameassistant, model_select, tools_Tigrao, vectorstore_in_agent)

        mensaxgem = f"""
        Crie o documento Pre-Projeto do projeto, salve e realize o upload no GitHub (usando autosave e autoupload) Baseie-se nas informacoes asseguir: \n  {mensagem}
        repo_name: \n
        {repo_name}\n
        token: \n
        {github_token}\n


        regra: em autoupload utilize o seguinte caminho relativo para o Dcumento Pre-Projeto: AppMap/PreProject/doc.txt
        """
        mensage3 = mensaxgem 
        response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensage3,
                                                                agent_id=AI_Tigrao, 
                                                                key=key,
                                                                app1=appfb,
                                                                client=client,
                                                                tools=tools_Tigrao,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions_Tigrao
                                                                )
                                                
                 
        ##Agent Destilation##                   
        Agent_destilation.DestilationResponseAgent(mensage3, response, instructionsassistant, nameassistant)
        
        path_name_doc_Pre_Projeto = os.getenv("PATH_NAME_DOC_PRE_PROJETO_ENV")
        return path_name_doc_Pre_Projeto










import threading
import subprocess
import os

def initchatconvert():
    # Definindo o diretório correto (onde está o )
    diretório_coreui = os.path.join(os.path.dirname(__file__), 'CoreUi', 'Chat')
    os.chdir(diretório_coreui)  # Mudando para o diretório 

    comando_terminal = ['python', 'Convert.py']  # Executando main.py dentro do diretório 
    subprocess.run(comando_terminal, shell=True)
initchatconvert()



#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI _init_environment_
from softwareai.CoreApp._init_environment_ import load_env


def get_file_sha(repo, path, token):
    url = f"https://api.github.com/repos/{repo}/contents/{path}"
    headers = {
        "Authorization": f"token {token}",
        "Accept": "application/vnd.github.v3+json"
    }
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.json()["sha"]
    return None

def autoupload(
                softwarepypath: str,
                repo_name: str,
                token: str
            ):
    

    
    directory = os.path.dirname(softwarepypath)

    with open(softwarepypath, "rb") as file:
        content = file.read()
        encoded_content = base64.b64encode(content).decode("utf-8")

    relative_path = os.path.relpath(softwarepypath, start=directory)  
    filename = os.path.basename(softwarepypath)
    github_path = relative_path.replace("\\", "/")  

    url = f"https://api.github.com/repos/A-I-O-R-G/{repo_name}/contents/{github_path}"

    response = requests.get(url, headers={"Authorization": f"token {token}"})
    sha = response.json().get("sha") if response.status_code == 200 else None

    data = {
        "message": f"Update {filename}",
        "content": encoded_content,
        "branch": "main"
    }
    if sha:
        data["sha"] = sha 

    put_response = requests.put(url, json=data, headers={"Authorization": f"token {token}"})
    if put_response.status_code in (200, 201):
        return {"status": "success", "message": f"Arquivo: {github_path} - Status: Sucesso ({put_response.status_code})"}
    else:
        return {"status": "success", "message": f"Erro ao enviar {github_path}: Status {put_response.status_code} - {put_response.text}"}

    




import typer
import os
from typing import Dict
import ast
import json
from transformers import AutoTokenizer, AutoModelForCausalLM
import torch
import time
import re
import subprocess

def extract_function_arguments(file_path):
    with open(file_path, "r", encoding="utf-8") as file:
        file_content = file.read()
    
    tree = ast.parse(file_content)
    function_args = {}

    for node in ast.walk(tree):
        print(node)
        if isinstance(node, ast.FunctionDef):
            arg_names = [arg.arg for arg in node.args.args]
            function_args[node.name] = arg_names
    
    return function_args

app = typer.Typer()

# Configurações de Chaves
@app.command()
def configure_db_company(
    namefordb: str = typer.Option(..., help="Nome para banco de dados."),
    databaseurl: str = typer.Option(..., help="URL do banco de dados."),
    storagebucketurl: str = typer.Option(..., help="URL do bucket de armazenamento."),
    pathkey: str = typer.Option(..., help="Caminho + arquivo com a Chave do banco de dados da companhia.")
    ):
        
    """
    Configura as credenciais do banco de dados da companhia.
    """

    file_Pathkey = os.path.join(pathkey)
    contentkey = None
    with open(file_Pathkey, 'r', encoding='utf-8') as file:
        contentkey = file.read()
        file.close()



    PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp/KeysFirebase'))
    file_path = os.path.join(PATH_caminho, f"keys.py")


    namefilter = namefordb.replace(" ", "_")

    with open(file_path, 'a', encoding='utf-8') as file:
        file.write(F'''
                
def keys_{namefilter}():
    key = {contentkey}
    credt = credentials.Certificate(key)
    app_{namefilter} = initialize_app(credt, {{
            'storageBucket': '{storagebucketurl}',
            'databaseURL': '{databaseurl}'
    }}, name='{namefilter}')
    return app_{namefilter}

        ''')
        file.close()



    # Lógica para armazenar ou validar a chave do banco de dados da companhia
    typer.echo(f"Chave do banco de dados da companhia Salva em: {file_path}")


@app.command()
def configure_db_app(
    namefordb: str = typer.Option(..., help="Nome para banco de dados."),
    databaseurl: str = typer.Option(..., help="URL do banco de dados."),
    storagebucketurl: str = typer.Option(..., help="URL do bucket de armazenamento."),
    pathkey: str = typer.Option(..., help="Caminho + arquivo com a Chave do banco de dados do app a ser governado.")
    ):
    """
    Configura as credenciais do banco de dados do aplicativo.
    """

    file_Pathkey = os.path.join(pathkey)
    contentkey = None
    with open(file_Pathkey, 'r', encoding='utf-8') as file:
        contentkey = file.read()
        file.close()

    PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp/KeysFirebase'))
    file_path = os.path.join(PATH_caminho, f"keys.py")

    namefilter = namefordb.replace(" ", "_")

    with open(file_path, 'a', encoding='utf-8') as file:
        file.write(F'''
                
def keys_{namefilter}():
    key = {contentkey}
    credt = credentials.Certificate(key)
    app_{namefilter} = initialize_app(credt, {{
            'storageBucket': '{storagebucketurl}',
            'databaseURL': '{databaseurl}'
    }}, name='{namefilter}')
    return app_{namefilter}

        ''')
        file.close()

    typer.echo(f"Chave do banco de dados do aplicativ Salva em: {file_path}")

@app.command()
def configure_openai(
    name: str = typer.Option(..., help="Nome para Chave da OpenAI."),
    key: str = typer.Option(..., help="Chave da OpenAI.")
    ):
    """
    Configura a chave de acesso da OpenAI.
    """

    PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp/KeysOpenAI'))
    file_path = os.path.join(PATH_caminho, f"keys.py")

    with open(file_path, 'a', encoding='utf-8') as file:
        file.write(F'''

class OpenAI_Keys_{name.replace(" ", "_")}:
    def keys():
        companyname = "{name.replace(" ", "_")}"
        str_key = "{key.replace(" ", "")}"
        return str_key
    


        ''')
        file.close()

    typer.echo(f"Chave da OpenAI Salva em: {file_path}")

@app.command()
def configure_huggingface(
                        name: str = typer.Option(..., help="Nome para Chave da Hugging Face."), 
                        key: str = typer.Option(..., help="Chave da Hugging Face.")
                        
                    ):
    """
    Configura a chave de acesso da Hugging Face.
    """

    PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp/KeysHuggingFace'))
    file_path = os.path.join(PATH_caminho, f"keys.py")

    with open(file_path, 'a', encoding='utf-8') as file:
        file.write(F'''

class HugKeys_{name.replace(" ", "_")}:
    def hug_{name.replace(" ", "_")}_keys():
        token = "{key}"
        return token

        ''')
        file.close()

    typer.echo(f"Chave da Hugging Face Salva em: {file_path}")

@app.command()
def configure_github_keys(
                        name: str = typer.Option(..., help="Nome para Chave do github."), 
                        github_username: str = typer.Option(..., help="Usuario do agente no github"),
                        github_token: str = typer.Option(..., help="Chave do agente no github")
                        
                    ):


    PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp/KeysGitHub'))
    file_path = os.path.join(PATH_caminho, f"keys.py")

    with open(file_path, 'a', encoding='utf-8') as file:
        file.write(F'''


class GithubKeys_{name.replace(" ", "_")}:
    def {name.replace(" ", "_")}_github_keys():
        github_username = "{github_username}"
        github_token = "{github_token}"
        return github_username, github_token


        ''')
        file.close()

    typer.echo(f"Chave do Github Salva em: {file_path}")




# Automatização da criacao de tools para agentes softwareai 
@app.command()
def create_function(
                pathfunction: str = typer.Option(..., help="Caminho + arquivo da função."),
                category: str = typer.Option(..., help="Categoria da função."),
                description_autogen_in_gpu: str = typer.Option(..., help="true"),
                cache_dir: str = typer.Option(..., help="D:/LLMModels"),
                ):
    """
    Cria uma nova função de maneira automatizada com base na descrição fornecida.
    """
    name = os.path.basename(pathfunction)
    name_filtrer = name.replace(" ", "_").replace(".py", "")
    category_filtrer = category.replace(" ", "_")
    
    #### Create tools  #### 
    args = extract_function_arguments(pathfunction)
    for function_name, arguments in args.items():
        def create_tool_file(tool_name, function_arguments, description="Descrição da função."):
            """Cria um arquivo de definição de tools baseado nos argumentos da função."""
            tools_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'CoreApp', 'SoftwareAI', 'Tools', f"{category_filtrer}"))
            os.makedirs(tools_dir, exist_ok=True)
            file_path = os.path.join(tools_dir, f"{tool_name}.py")
            # Criar propriedades baseadas nos argumentos da função
            properties = {arg: {"type": "string", "description": f"Descrição do argumento {arg}."} for arg in function_arguments}
            
            if description_autogen_in_gpu:
                with open(os.path.abspath(os.path.join(os.path.dirname(__file__), 'CoreApp', 'SoftwareAI', 'Functions', f"{category_filtrer}", f"{name_filtrer}.py" )), "r", encoding="utf-8") as f:
                    contentcode = f.read()
                    f.close()

                def autocreatedescription(code):

                    
                    model_name = "deepseek-ai/DeepSeek-R1-Distill-Qwen-1.5B"
                    max_new_tokens = 1900

                    input_text = f""" 
                Create a description for the code function arguments

                code:
                {code}

                    """

                    # Carregar o modelo e o tokenizer
                    model = AutoModelForCausalLM.from_pretrained(
                        model_name,
                        torch_dtype="auto",
                        device_map="auto",
                        cache_dir=cache_dir
                    )
                    tokenizer = AutoTokenizer.from_pretrained(model_name, cache_dir=cache_dir)

                    # Preparar a entrada
                    messages = [{"role": "user", "content": f"{input_text}"}]
                    text = tokenizer.apply_chat_template(messages, tokenize=False, add_generation_prompt=True)

                    model_inputs = tokenizer([text], return_tensors="pt").to(model.device)

                    # Inicializar variáveis
                    input_ids = model_inputs['input_ids']
                    generated_ids = input_ids
                    num_generated_tokens = 0

                    # Variável para armazenar o texto após a segunda ocorrência
                    text_after_second_think = None
                    found_second_think = False

                    # Variável para acumular texto gerado no streaming
                    current_text = ""

                    # Gerar texto em "streaming"
                    while num_generated_tokens < max_new_tokens:
                        # Gerar um token de cada vez
                        outputs = model.generate(
                            generated_ids,
                            max_length=generated_ids.shape[1] + 1,  # Apenas um token adicional
                            do_sample=True,
                            top_k=50,
                            top_p=0.95,
                            temperature=0.7,
                            pad_token_id=tokenizer.eos_token_id
                        )
                        
                        # Atualizar a sequência gerada
                        generated_ids = outputs[:, :generated_ids.shape[1] + 1]
                        generated_token = generated_ids[0, -1].item()
                        
                        # Mostrar o token gerado
                        generated_text = tokenizer.decode(generated_ids[0, -1:], skip_special_tokens=True)
                        print(generated_text, end='', flush=True)
                        
                        # Acumular texto até a segunda ocorrência de </think>
                        current_text += generated_text

                        # Verificar se o token gerado é o final (EOS)
                        if generated_token == tokenizer.eos_token_id:
                            
                            break

                    matches = list(re.finditer(r"</think>\n", current_text))
                    #print(len(matches))
                    if len(matches) == 1:
                        start_index = matches[0].end()
                        text_after_second_think = current_text[start_index:]
                        return text_after_second_think


                description_DeepSeek_R1 = autocreatedescription(contentcode)

                description = description_DeepSeek_R1
                # Criar propriedades baseadas nos argumentos da função
                properties = {arg: {"type": "string", "description": f"Descrição do argumento {arg}."} for arg in function_arguments}
                
                
            required_args = function_arguments

            # Criar o conteúdo do arquivo
            tool_content = {
                "type": "function",
                "function": {
                    "name": tool_name,
                    "description": description,
                    "parameters": {
                        "type": "object",
                        "properties": properties,
                        "required": required_args
                    }
                }
            }
            
            
            # Escrever no arquivo
            with open(file_path, 'w', encoding='utf-8') as file:
                file.write(f"tools_{tool_name} = [\n")
                file.write(json.dumps(tool_content, indent=4))
                file.write("\n]\n")

        create_tool_file(tool_name=function_name, function_arguments=arguments)
        break

    ################################ 

    #### Create init functions  #### 
    with open(os.path.join(os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp')), f"_init_functions_.py"), 'r+', encoding='utf-8') as file:
        content = file.read()
        if f"from softwareai.CoreApp.SoftwareAI.Functions.{category_filtrer}.{name_filtrer}" not in content:
            file.write(F'''
from softwareai.CoreApp.SoftwareAI.Functions.{category_filtrer}.{name_filtrer} import *
            ''')
            file.close()

    ################################ 
    



    #### Create init tools #### 
    with open(os.path.join(os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp' )), f"_init_tools_.py"), 'r+', encoding='utf-8') as file:
        content = file.read()
        if f"{category_filtrer}.{name_filtrer}" not in content:
            
            file.write(F'''
from softwareai.CoreApp.SoftwareAI.Tools.{category_filtrer}.{name_filtrer} import *
            ''')
            file.close()

    ################################ 



    #### Create Submit Output #### 
    PATH_caminho_Submit_Output = os.path.abspath(os.path.join(os.path.dirname(__file__), f'CoreApp', 'SoftwareAI', 'Functions_Submit_Outputs', f'{category_filtrer}'))
    os.makedirs(PATH_caminho_Submit_Output, exist_ok=True)
    file_path_Submit_Output = os.path.join(PATH_caminho_Submit_Output, f'{name_filtrer}.py')
    with open(file_path_Submit_Output, 'x', encoding='utf-8') as file:
        file.write(F'''

#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI Functions
from softwareai.CoreApp._init_functions_ import *
#########################################

tool_outputs = []
def submit_output_{name_filtrer}(function_name,
                                function_arguments,
                                tool_call,
                                threead_id,
                                client,
                                run,
                                appfb,
                                appproduct
                                ):

    global tool_outputs
    
    # Mapear funções pelo nome
    functions_map = {{
        "{name_filtrer}": {name_filtrer},

    }}

    # Obter a função correspondente
    target_function = functions_map.get(function_name)
    if not target_function:
        print(f"Função {{function_name}} não encontrada.")
        return False

    # Inspecionar os argumentos da função
    function_signature = inspect.signature(target_function)
    function_parameters = function_signature.parameters

    # Preparar argumentos para chamada
    args = json.loads(function_arguments)
    call_arguments = {{}}

    # Adicionar parâmetros obrigatórios 
    if "appcompany" in function_parameters:
        call_arguments["appcompany"] = appfb
    if "appfb" in function_parameters:
        call_arguments["appfb"] = appfb
    if "appproduct" in function_parameters:
        call_arguments["appproduct"] = appproduct
    if "app_product" in function_parameters:
        call_arguments["app_product"] = appproduct
        
    # Adicionar outros argumentos do JSON somente se estiverem na assinatura da função
    for arg_name, arg_value in args.items():
        if arg_name in function_parameters:
            call_arguments[arg_name] = arg_value

    try:
        # Chamar a função com os argumentos preparados
        result = target_function(**call_arguments)

        # Submeter o resultado
        tool_call_id = tool_call.id
        client.beta.threads.runs.submit_tool_outputs(
            thread_id=threead_id,
            run_id=run.id,
            tool_outputs=[
                {{
                    "tool_call_id": tool_call_id,
                    "output": json.dumps(result),
                }}
            ]
        )
        print("Tool outputs submitted successfully.")
        return True

    except Exception as e:
        print(f"Erro ao executar  {{function_name}}: {{e}}")
        
        ''')
        file.close()
   
    ################################ 


    #### Create init submit outputs  #### 
    new_function_name = f"submit_output_{name_filtrer}"
    with open(os.path.join(os.path.abspath(os.path.join(os.path.dirname(__file__), 'CoreApp')), '_init_submit_outputs_.py'), 'r+', encoding='utf-8') as file:
        content = file.read()
        if new_function_name not in content:
            updated_content = content.replace(
                "functions_to_call = [",
                f"functions_to_call = [\n        {new_function_name},"
            )
            file.seek(0)
            file.write(updated_content)
            file.truncate()
        else:
            print("Função já está na lista.")
    with open(os.path.join(os.path.abspath(os.path.join(os.path.dirname(__file__), 'CoreApp')), '_init_submit_outputs_.py'), 'r+', encoding='utf-8') as file:
        content = file.read()
        new_import = f"from softwareai.CoreApp.SoftwareAI.Functions_Submit_Outputs.{category_filtrer}.{name_filtrer} import submit_output_{name_filtrer}\n"
        if new_import not in content:
            file.seek(0)
            file.write(new_import + content)
            file.truncate()

    ################################ 

    
    file_path_tool = os.path.abspath(os.path.join(os.path.dirname(__file__), 'CoreApp', 'SoftwareAI', 'Tools', f"{category_filtrer}", f"{name_filtrer}.py"))
    typer.echo(f"\nTool criada e salva em: {file_path_tool}")






# Agente Selecionado Em Modo API
@app.command()
def select_agent_mode_api(
    name_agent: str = typer.Option(..., help="Nome do agente a ser executado"),
    category_agent: str = typer.Option(..., help="Nome da Categoria do agente a ser executado"),
    local_execute_port: str = typer.Option(..., help="A porta onde o agente sera consultado"), 
    ):

    name_agent_filter = name_agent.replace(" ", "_").replace(".py", "")
    category_agent_filter = category_agent.replace(" ", "_")


    # Caminho do arquivo
    diretorio = os.path.join(
        os.path.dirname(__file__),
        'CoreApp',
        'Agents',
        f"{category_agent_filter}",
        f'{name_agent_filter}_api.py'
    )
    with open(diretorio, 'r') as arquivo:
        conteudo = arquivo.readlines()

    # Modifica a porta na linha contendo app.run
    for i, linha in enumerate(conteudo):
        if "if __name__ == '__main__':" in linha:
            # Procura a linha com app.run na sequência
            for j in range(i + 1, len(conteudo)):
                if "app.run" in conteudo[j]:
                    conteudo[j] = f"    app.run(port={local_execute_port})\n"
                    break
            break

    # Escreve de volta no arquivo
    with open(diretorio, 'w') as arquivo:
        arquivo.writelines(conteudo)

    print(f"A porta foi alterada para {local_execute_port}")
    
    comando_terminal = ['start', 'python', diretorio]

    subprocess.Popen(comando_terminal, shell=True)
    print(f"Executando {name_agent} em http://127.0.0.1:100")



if __name__ == "__main__":
    app()




#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Functions
from softwareai.CoreApp._init_functions_ import *
#########################################

def submit_output_execute_py(function_name,
                                function_arguments,
                                tool_call,
                                threead_id,
                                client,
                                run
                                ):


    if function_name == 'execute_py':
        args = json.loads(function_arguments)
        result = execute_py(
            filepath=args['filepath']
        )
        tool_call_id = tool_call.id
        client.beta.threads.runs.submit_tool_outputs(
            thread_id=threead_id,
            run_id=run.id,
            tool_outputs=[
                {
                    "tool_call_id": tool_call_id,
                    "output": json.dumps(result)
                }
            ]
        )




import threading
import subprocess
import os
def initeditor():
    diretório_coreui = os.path.join(os.path.dirname(__file__), 'CoreUi', 'Editor')
    os.chdir(diretório_coreui)
    comando_terminal = ['python', 'main.py'] 
    subprocess.run(comando_terminal, shell=True)
initeditor()



def get_current_datetime():
    import datetime
    current_datetime = datetime.datetime.now()
    formatted_datetime = current_datetime.strftime('%Y-%m-%d %H:%M:%S')
    return f"{formatted_datetime}"



############# Update Software Repo trontechnicalanalysistool Everyday ############################

# IMPORT SoftwareAI Agents
from softwareai.CoreApp._init_agents_ import AgentInitializer
#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################

name_app = "appx"
appfb = FirebaseKeysinit._init_app_(name_app)

def Update(SoftwareRepoUrl, numberUpdate=5, timebetweenupdate=300, lang="pt"):
    for i in range(numberUpdate):
        byte_manager = AgentInitializer.get_agent('ByteManager') 
        if lang == "pt":
            mensagem = f"solicito uma atualização do repositorio {SoftwareRepoUrl}"
        elif lang == "en":
            mensagem = f"request a repository update {SoftwareRepoUrl}"
        owner_response = byte_manager.AI_1_ByteManager_Company_Owners(mensagem, appfb)
        print(owner_response)
        time.sleep(timebetweenupdate)

schedule.every().day.at("14:23").do(lambda: Update("https://github.com/SoftwareAI-Company/NordVpnAutoRotate", 4))
while True:
    schedule.run_pending()
    time.sleep(1)




#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI All Paths 
from softwareai.CoreApp._init_paths_ import *
#########################################
# IMPORT SoftwareAI Instructions
from softwareai.CoreApp._init_Instructions_ import *
#########################################
# IMPORT SoftwareAI Tools
from softwareai.CoreApp._init_tools_ import *
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *
#########################################





class Gerente_de_projeto:
    def __init__(self):
        pass

    ##############################################################################################
    def Bob_Gerente_de_projeto(self, appfb, client, path_name_doc_Pre_Projeto, repo_name, UseVectorstoreToGenerateFiles = True):

        key = "AI_Bob_Gerente_de_Projeto"
        nameassistant = "AI Bob Gerente de Projeto"
        model_select = "gpt-4o-mini-2024-07-18"
        
        Upload_1_file_in_thread = None
        Upload_1_file_in_message = None
        Upload_1_image_for_vision_in_thread = None
        vectorstore_in_assistant = None
        vectorstore_in_Thread = None
        Upload_list_for_code_interpreter_in_thread = None

        
        key_openai = OpenAIKeysteste.keys()

        github_username, github_token = GithubKeys.BobGerenteDeProjeto_github_keys()
        
        path_nome_do_cronograma = os.getenv("PATH_NOME_DO_CRONOGRAMA_ENV")
        

        AI_Bob, instructionsassistant, nameassistant, model_select = AutenticateAgent.create_or_auth_AI(appfb, client, key, instructionBob, nameassistant, model_select, tools_Bob, vectorstore_in_assistant)
        if UseVectorstoreToGenerateFiles == True:
            file_paths = [os.path.abspath(os.path.join(os.path.dirname(__file__), "../../", f"environment.txt")), path_name_doc_Pre_Projeto]
            AI_Bob = Agent_files_update.del_all_and_upload_files_in_vectorstore(appfb, client, AI_Bob, "Bob_Work_Environment", file_paths)

        
        mensagem = f"""
        Crie o cronograma do projeto, salve e realize o upload no GitHub (usando autosave e autoupload) Baseie-se no documento pre projeto armazenado em `Bob_Work_Environment`
        repo_name: \n
        {repo_name}\n
        token: \n
        {github_token}\n


        regra: em autoupload utilize o seguinte caminho relativo para o cronograma AppMap/SpreadsheetAndTimeline
        """

        adxitional_instructions_Bob = f"""
        estrutura do projeto esta armazenada em environment.txt
        """
               
        response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensagem,
                                                                agent_id=AI_Bob, 
                                                                key=key,
                                                                app1=appfb,
                                                                client=client,
                                                                tools=tools_Bob,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions_Bob
                                                                )
    
                                            
        ##Agent Destilation##                   
        Agent_destilation.DestilationResponseAgent(mensagem, response, instructionsassistant, nameassistant)
        

        if UseVectorstoreToGenerateFiles == True:
            file_paths = [path_name_doc_Pre_Projeto, path_nome_do_cronograma]
            AI_Bob = Agent_files_update.del_all_and_upload_files_in_vectorstore(appfb, client, AI_Bob, "Bob_Work_Environment", file_paths)
            mensagem = f"""
            Crie o planilha do projeto, salve e realize o upload no GitHub (usando autosave e autoupload) Baseie-se no documento pre projeto e cronograma armazenado em `Bob_Work_Environment`
            repo_name: \n
            {repo_name}\n
            token: \n
            {github_token}\n
            """


        adxitional_instructions_Bob = f"""
        estrutura do projeto esta armazenada em environment.txt
        """

        response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensagem,
                                                                agent_id=AI_Bob, 
                                                                key=key,
                                                                app1=appfb,
                                                                client=client,
                                                                tools=tools_Bob,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions_Bob
                                                                )
    
                                            
        ##Agent Destilation##                   
        Agent_destilation.DestilationResponseAgent(mensagem, response, instructionsassistant, nameassistant)
        



        return os.getenv("PATH_PLANILHA_PROJETO_ENV"), path_nome_do_cronograma, path_name_doc_Pre_Projeto







tools_SignalMaster = [
    {"type": "file_search"},
    {
        "type": "function",
        "function": {
            "name": "improve_code_and_create_pull_request",
            "description": "Realiza melhorias no código e cria um pull request no repositório GitHub.",
            "parameters": {
                "type": "object",
                "properties": {
                    "repo_owner": {
                        "type": "string",
                        "description": "Nome do dono do repositório no GitHub."
                    },
                    "repo_name": {
                        "type": "string",
                        "description": "Nome do repositório no GitHub."
                    },
                    "branch_name": {
                        "type": "string",
                        "description": "Nome da branch onde o código será atualizado."
                    },
                    "file_path": {
                        "type": "string",
                        "description": "Caminho do arquivo no repositório."
                    },
                    "commit_message": {
                        "type": "string",
                        "description": "Mensagem de commit descrevendo as melhorias."
                    },
                    "improvements": {
                        "type": "string",
                        "description": "Novo código melhorado."
                    },
                    "pr_title": {
                        "type": "string",
                        "description": "Titulo do Pull request."
                    },
                    "token": {
                        "type": "string",
                        "description": "Token de autenticação do GitHub."
                    }
                },
                "required": ["repo_owner", "repo_name", "branch_name", "file_path", "commit_message", "improvements", "pr_title",  "token"]
            }
        }
    }

]




# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI All Paths 
load_dotenv(dotenv_path=os.path.join(os.path.dirname(os.path.abspath(__file__)), "environment.env"))
load_dotenv(find_dotenv('environment.env'), override=True)
#########################################
